import { __decorate, __param, __extends, __awaiter, __generator, __read, __assign, __spread } from 'tslib';
import { ElementRef, HostListener, Component, ChangeDetectionStrategy, ViewEncapsulation, Optional, Host, Input, Directive, InjectionToken, EventEmitter, ViewChild, Output, forwardRef, ContentChild, Self, HostBinding, Inject, TemplateRef, ViewContainerRef, Renderer2, LOCALE_ID, Pipe, ChangeDetectorRef, NgZone, NgModule } from '@angular/core';
import { trigger, transition, style, animate } from '@angular/animations';
import { BehaviorSubject, fromEvent, merge, EMPTY, combineLatest, of } from 'rxjs';
import { mixinDisabled, mixinGrouped, ImageConverter, mixinFocused, mixinReadonly, mixinRequired, mixinError, mixinHideErrors, coerceBooleanProperty, Platform, mixinLoad } from '@bravobit/bb-foundation';
import { NG_VALUE_ACCESSOR, NgControl, NG_VALIDATORS, FormGroupDirective } from '@angular/forms';
import { map, mapTo, startWith, distinctUntilChanged, share, tap, delay } from 'rxjs/operators';
import { Localize, LocalizeModule } from '@bravobit/bb-foundation/localize';
import { DatePipe, CommonModule } from '@angular/common';
import { DomSanitizer } from '@angular/platform-browser';

var BbDropdown = /** @class */ (function () {
    function BbDropdown(_elementRef) {
        this._elementRef = _elementRef;
        // State.
        this.visible$ = new BehaviorSubject(false);
    }
    BbDropdown.prototype.toggle = function () {
        return this.visible$.next(!this.visible$.getValue());
    };
    BbDropdown.prototype.close = function () {
        return this.visible$.next(false);
    };
    BbDropdown.prototype.onDocumentClick = function (target) {
        if (!target) {
            return;
        }
        if (this._elementRef.nativeElement.contains(target)) {
            return;
        }
        return this.close();
    };
    BbDropdown.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        HostListener('document:click', ['$event.target'])
    ], BbDropdown.prototype, "onDocumentClick", null);
    BbDropdown = __decorate([
        Component({
            selector: 'bb-dropdown',
            template: "<button (click)=\"toggle()\"\r\n        [class.opened]=\"visible$ | async\"\r\n        type=\"button\">\r\n    <ng-content></ng-content>\r\n</button>\r\n\r\n<div *ngIf=\"visible$ | async\"\r\n     [@dropdownMenuAnimation]=\"true\"\r\n     class=\"bb-dropdown-menu\"\r\n     role=\"list\">\r\n    <ng-content select=\"[bbDropdownItem], hr\"></ng-content>\r\n</div>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            host: { 'class': 'bb-dropdown' },
            preserveWhitespaces: false,
            animations: [
                trigger('dropdownMenuAnimation', [
                    transition(':enter', [
                        // 1. Set the initial state.
                        style({ opacity: 0, transform: 'scale(0.9)' }),
                        // 2. Start the animation to show the item.
                        animate('80ms cubic-bezier(0, 0, .2, 1)', style({ opacity: 1, transform: 'scale(1)' }))
                    ]),
                    transition(':leave', [
                        // 1. Set the initial state.
                        style({ opacity: 1, transform: 'scale(1)' }),
                        // 2. Start the animation to hide the item.
                        animate('80ms cubic-bezier(0, 0, .2, 1)', style({ opacity: 0, transform: 'scale(0.9)' }))
                    ])
                ])
            ],
            styles: [".bb-dropdown{position:relative;display:inline-block}.bb-dropdown.right>.bb-dropdown-menu{right:0;left:auto;-ms-transform-origin:top right;transform-origin:top right}.bb-dropdown-menu{left:0;top:100%;padding:0;z-index:10;display:block;margin:10px 0 0;border-radius:4px;position:absolute;list-style-type:none;background-color:#fff;border:1px solid #ccc;-ms-transform-origin:top left;transform-origin:top left;box-shadow:0 0 6px 0 rgba(0,0,0,.1)}.bb-dropdown-menu:after{width:0;top:-5px;height:0;right:5px;content:\"\";position:absolute;border-style:solid;border-width:0 10px 5px;border-color:transparent transparent #d9d9d9}.bb-dropdown-menu>hr{margin:0;height:1px;border:none;background-color:#ccc}.bb-dropdown-item{width:100%;border:none;line-height:1;display:block;cursor:pointer;text-align:left;font-weight:500;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;padding:12px 20px;white-space:nowrap;text-decoration:none;color:#676767!important;background-color:transparent;transition:background-color 250ms cubic-bezier(0,0,.2,1)}.bb-dropdown-item:first-child{border-top-left-radius:4px;border-top-right-radius:4px}.bb-dropdown-item:hover{background-color:#f6f6f6}.bb-dropdown-item:focus{box-shadow:0 0 0 3px rgba(220,224,233,.5)}.bb-dropdown-item:active{background-color:#f0f0f0}"]
        })
    ], BbDropdown);
    return BbDropdown;
}());
var BbDropdownItem = /** @class */ (function () {
    function BbDropdownItem(_parent) {
        this._parent = _parent;
        // Inputs.
        this.closeOnClick = true;
    }
    BbDropdownItem.prototype.onClick = function () {
        if (!this.closeOnClick) {
            return;
        }
        this._parent && this._parent.close();
    };
    BbDropdownItem.ctorParameters = function () { return [
        { type: BbDropdown, decorators: [{ type: Optional }, { type: Host }] }
    ]; };
    __decorate([
        Input()
    ], BbDropdownItem.prototype, "closeOnClick", void 0);
    __decorate([
        HostListener('click')
    ], BbDropdownItem.prototype, "onClick", null);
    BbDropdownItem = __decorate([
        Directive({
            selector: '[bbDropdownItem]',
            host: { 'class': 'bb-dropdown-item' }
        }),
        __param(0, Optional()), __param(0, Host())
    ], BbDropdownItem);
    return BbDropdownItem;
}());

var ELEMENTS_ICONS = new InjectionToken('elements_icons');
var ELEMENTS_ERRORS = new InjectionToken('elements_errors');

var BbImagePickerBase = /** @class */ (function () {
    function BbImagePickerBase() {
    }
    return BbImagePickerBase;
}());
var BbImagePickerMixinBase = mixinDisabled(mixinGrouped(BbImagePickerBase));
var BbImagePicker = /** @class */ (function (_super) {
    __extends(BbImagePicker, _super);
    function BbImagePicker(_converter) {
        var _this = _super.call(this) || this;
        _this._converter = _converter;
        // Inputs.
        _this.label = null;
        _this.placeholder = null;
        _this.styleDimensions = null;
        _this.cropDimensions = null;
        // Form.
        _this._value$ = new BehaviorSubject(null);
        _this.valueChange = new EventEmitter();
        _this.onTouchedCallback = function () {
        };
        _this.onChangeCallback = function () {
        };
        return _this;
    }
    BbImagePicker_1 = BbImagePicker;
    Object.defineProperty(BbImagePicker.prototype, "value", {
        get: function () {
            return this._value$.getValue();
        },
        set: function (value) {
            this._value$.next(value);
            this.valueChange.emit(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbImagePicker.prototype, "image", {
        get: function () {
            var _this = this;
            return this._value$.pipe(map(function (value) { return value || _this.placeholder; }), map(function (value) { return value ? "url(" + value + ")" : null; }));
        },
        enumerable: true,
        configurable: true
    });
    BbImagePicker.prototype.onFileChange = function (event) {
        return __awaiter(this, void 0, void 0, function () {
            var file, _a, error_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        file = event.target.files[0];
                        // Check if the file exists.
                        if (!file || this.disabled) {
                            return [2 /*return*/];
                        }
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, 3, , 4]);
                        _a = this;
                        return [4 /*yield*/, this._converter.toDataUri(file, this.cropDimensions)];
                    case 2:
                        _a.value = _b.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _b.sent();
                        this.value = null;
                        return [3 /*break*/, 4];
                    case 4:
                        this.onChangeCallback(this.value);
                        if (this.fileInput && this.fileInput.nativeElement) {
                            this.fileInput.nativeElement.value = '';
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    BbImagePicker.prototype.openFilePicker = function () {
        if (this.disabled) {
            return;
        }
        // Execute a click on the file input.
        this.fileInput
            && this.fileInput.nativeElement
            && this.fileInput.nativeElement.click
            && this.fileInput.nativeElement.click();
    };
    BbImagePicker.prototype.writeValue = function (value) {
        this.value = value;
    };
    BbImagePicker.prototype.registerOnChange = function (method) {
        this.onChangeCallback = method;
    };
    BbImagePicker.prototype.registerOnTouched = function (method) {
        this.onTouchedCallback = method;
    };
    BbImagePicker.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    var BbImagePicker_1;
    BbImagePicker.ctorParameters = function () { return [
        { type: ImageConverter }
    ]; };
    __decorate([
        ViewChild('fileInput', { static: true })
    ], BbImagePicker.prototype, "fileInput", void 0);
    __decorate([
        Input()
    ], BbImagePicker.prototype, "label", void 0);
    __decorate([
        Input()
    ], BbImagePicker.prototype, "placeholder", void 0);
    __decorate([
        Input()
    ], BbImagePicker.prototype, "styleDimensions", void 0);
    __decorate([
        Input()
    ], BbImagePicker.prototype, "cropDimensions", void 0);
    __decorate([
        Input()
    ], BbImagePicker.prototype, "value", null);
    __decorate([
        Output()
    ], BbImagePicker.prototype, "valueChange", void 0);
    BbImagePicker = BbImagePicker_1 = __decorate([
        Component({
            selector: 'bb-image-picker',
            template: "<!-- The input that can open the file picker. -->\r\n<input #fileInput\r\n       (change)=\"onFileChange($event)\"\r\n       class=\"bb-image-picker-input\"\r\n       type=\"file\"\r\n       tabindex=\"-1\"\r\n       accept=\"image/*\">\r\n\r\n<span [style.width]=\"styleDimensions?.width || '100%'\"\r\n      [style.height]=\"styleDimensions?.height || '215px'\"\r\n      [style.background-image]=\"image | async\"\r\n      class=\"bb-image-input-image\">\r\n    </span>\r\n\r\n<button *ngIf=\"label as labelText\"\r\n        (click)=\"openFilePicker()\"\r\n        bb-button\r\n        class=\"bb-image-input-button\"\r\n        type=\"button\">\r\n    {{ labelText }}\r\n</button>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            host: {
                'class': 'bb-image-picker',
                '[class.disabled]': 'disabled',
                '[class.grouped]': 'grouped'
            },
            inputs: ['grouped', 'disabled'],
            providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return BbImagePicker_1; }), multi: true }],
            preserveWhitespaces: false,
            styles: [".bb-image-picker{display:-ms-flexbox;display:flex;-ms-flex-align:center;align-items:center;-ms-flex-direction:column;flex-direction:column}.bb-image-picker.circle>.bb-image-input-image{border-radius:50%}.bb-image-picker.grouped{margin-bottom:20px}.bb-image-picker.disabled{opacity:.5;cursor:default;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;pointer-events:none}.bb-image-picker-input{opacity:0;z-index:-1;width:.1px;height:.1px;overflow:hidden;position:absolute}.bb-image-input-image{display:-ms-flexbox;display:flex;cursor:default;position:relative;border-radius:4px;-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center;background:url(data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7) center center/cover no-repeat rgba(0,0,0,.2)}.bb-image-input-button{margin-top:10px}"]
        })
    ], BbImagePicker);
    return BbImagePicker;
}(BbImagePickerMixinBase));

var nextUniqueId = 0;
var BbFormControlBase = /** @class */ (function () {
    function BbFormControlBase() {
    }
    return BbFormControlBase;
}());
var BbFormControlMixinBase = mixinDisabled(mixinFocused(mixinReadonly(mixinGrouped(mixinRequired(mixinError(mixinHideErrors(BbFormControlBase)))))));
var BbFormControl = /** @class */ (function (_super) {
    __extends(BbFormControl, _super);
    function BbFormControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // Inputs.
        _this.label = null;
        _this.hint = null;
        // Data.
        _this.arrow = false;
        // Data.
        _this.labelId = "bb-form-control-" + nextUniqueId++;
        return _this;
    }
    BbFormControl.prototype.onErrorChange = function (error) {
        this.error = !!error;
    };
    __decorate([
        ContentChild(NgControl)
    ], BbFormControl.prototype, "ngControl", void 0);
    __decorate([
        Input()
    ], BbFormControl.prototype, "label", void 0);
    __decorate([
        Input()
    ], BbFormControl.prototype, "hint", void 0);
    BbFormControl = __decorate([
        Component({
            selector: 'bb-form-control',
            template: "<!-- Label of the form control. -->\r\n<label *ngIf=\"label as labelContent\"\r\n       [attr.for]=\"labelId\"\r\n       class=\"bb-form-control-label\">\r\n    <ng-template [bbTemplate]=\"labelContent\">\r\n        {{ labelContent }}\r\n    </ng-template>\r\n</label>\r\n\r\n<!-- The input with the addons. -->\r\n<div class=\"bb-form-control-container\">\r\n    <ng-content select=\"[bbPrefix]\"></ng-content>\r\n    <ng-content select=\"[bbInput]\"></ng-content>\r\n    <ng-content select=\"[bbSuffix]\"></ng-content>\r\n</div>\r\n\r\n<!-- The error component. -->\r\n<bb-form-error *ngIf=\"!hideErrors\"\r\n               [control]=\"ngControl\"\r\n               (errorChange)=\"onErrorChange($event)\">\r\n</bb-form-error>\r\n\r\n<!-- The form control hint. -->\r\n<p *ngIf=\"hint as hintContent\"\r\n   class=\"bb-form-control-hint\">\r\n    <ng-template [bbTemplate]=\"hintContent\">\r\n        {{ hintContent }}\r\n    </ng-template>\r\n</p>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            host: {
                'class': 'bb-form-control',
                '[class.readonly]': 'readonly',
                '[class.required]': 'required',
                '[class.disabled]': 'disabled',
                '[class.focused]': 'focused',
                '[class.grouped]': 'grouped',
                '[class.arrow]': 'arrow',
                '[class.error]': 'error'
            },
            inputs: ['grouped', 'hideErrors'],
            preserveWhitespaces: false,
            styles: [".bb-form-control{color:#2d3c4d;display:block}.bb-form-control>.bb-form-control-container>.bb-prefix,.bb-form-control>.bb-form-control-container>.bb-suffix{margin:0 8px;color:#b4b4b4;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control.grouped{margin-bottom:20px}.bb-form-control.small{max-width:240px}.bb-form-control.medium{max-width:480px}.bb-form-control.large{max-width:720px}.bb-form-control.required>.bb-form-control-label:after{content:\"*\";color:#c23934;font-size:12px;line-height:1.5}.bb-form-control.disabled>.bb-form-control-container{cursor:default;pointer-events:none;background-color:rgba(204,204,204,.64)}.bb-form-control.disabled>.bb-form-control-container>input,.bb-form-control.disabled>.bb-form-control-container>select,.bb-form-control.disabled>.bb-form-control-container>textarea{color:#aaa}.bb-form-control.readonly>.bb-form-control-container{cursor:default;border:1px dotted #bdc4c9}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container{background-color:#fff;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(0,123,255,.2)}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>input::-moz-placeholder,.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>textarea::-moz-placeholder{color:#b4b4b4;-moz-transition:color .3s cubic-bezier(0,0,.2,1);transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>input:-ms-input-placeholder,.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>textarea:-ms-input-placeholder{color:#b4b4b4;-ms-transition:color .3s cubic-bezier(0,0,.2,1);transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>input::-ms-input-placeholder,.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>textarea::-ms-input-placeholder{color:#b4b4b4;-ms-transition:color .3s cubic-bezier(0,0,.2,1);transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>input::placeholder,.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>textarea::placeholder{color:#b4b4b4;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control.error>.bb-form-control-label{color:#c23934}.bb-form-control.error>.bb-form-control-hint{display:none}.bb-form-control.error>.bb-form-control-container{border:1px solid #962b26;background-color:rgba(194,57,52,.25)!important;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(194,57,52,.4)!important}.bb-form-control.error>.bb-form-control-container>.bb-prefix,.bb-form-control.error>.bb-form-control-container>.bb-suffix{color:#c23934}.bb-form-control.error>.bb-form-control-container>input::-moz-placeholder,.bb-form-control.error>.bb-form-control-container>textarea::-moz-placeholder{color:rgba(194,57,52,.4)!important}.bb-form-control.error>.bb-form-control-container>input:-ms-input-placeholder,.bb-form-control.error>.bb-form-control-container>textarea:-ms-input-placeholder{color:rgba(194,57,52,.4)!important}.bb-form-control.error>.bb-form-control-container>input::-ms-input-placeholder,.bb-form-control.error>.bb-form-control-container>textarea::-ms-input-placeholder{color:rgba(194,57,52,.4)!important}.bb-form-control.error>.bb-form-control-container>input::placeholder,.bb-form-control.error>.bb-form-control-container>textarea::placeholder{color:rgba(194,57,52,.4)!important}.bb-form-control.arrow>.bb-form-control-container:after{width:20px;content:\"\";height:36px;display:-ms-flexbox;display:flex;min-width:20px;background-repeat:no-repeat;background-position:calc(100% - 8px) center;background-image:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA0AAAAGCAYAAAAYLBS/AAAACXBIWXMAAAsSAAALEgHS3X78AAAAWUlEQVQYlY3QsQ2DMAAEwMMTsAkrZInvWY3WXiIjJJtkA1OBRCIRf/nSffFTrXXGE4uxbCXJBw+8R0CStcAg3JKsUI7mDzzBBd3AC4Cp9/4z21o7znl9A9gBAf4fpjLTwkUAAAAASUVORK5CYII=)}.bb-form-control-container{width:100%;display:-ms-flexbox;display:flex;overflow:hidden;position:relative;border-radius:3px;-ms-flex-align:center;align-items:center;border:1px solid #b6bbc1;background-color:#fff8ff;transition-duration:250ms;transition-property:background-color,box-shadow;transition-timing-function:cubic-bezier(0,0,.2,1);box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1)}.bb-form-control-container>input::-moz-placeholder,.bb-form-control-container>textarea::-moz-placeholder{color:#d2d2d2;-moz-transition:color .3s cubic-bezier(0,0,.2,1);transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control-container>input:-ms-input-placeholder,.bb-form-control-container>textarea:-ms-input-placeholder{color:#d2d2d2;-ms-transition:color .3s cubic-bezier(0,0,.2,1);transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control-container>input::-ms-input-placeholder,.bb-form-control-container>textarea::-ms-input-placeholder{color:#d2d2d2;-ms-transition:color .3s cubic-bezier(0,0,.2,1);transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control-container>input::placeholder,.bb-form-control-container>textarea::placeholder{color:#d2d2d2;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control-container>input,.bb-form-control-container>select,.bb-form-control-container>textarea{margin:0;width:100%;border:none;color:#3d464d;font-size:16px;-webkit-appearance:none;-moz-appearance:none;appearance:none;background-color:transparent}.bb-form-control-container>input,.bb-form-control-container>select{height:36px;padding:0 8px}.bb-form-control-container>select::-ms-expand{display:none}.bb-form-control-container>input::-ms-clear,.bb-form-control-container>input::-ms-reveal{width:0;height:0;display:none}.bb-form-control-container>input::-webkit-search-cancel-button,.bb-form-control-container>input::-webkit-search-decoration,.bb-form-control-container>input::-webkit-search-results-button,.bb-form-control-container>input::-webkit-search-results-decoration{display:none}.bb-form-control-container>textarea{resize:none;padding:8px}.bb-form-control-label{color:#000;display:block;position:relative;margin-bottom:4px;transition:color 30ms cubic-bezier(0,0,.2,1)}.bb-form-control-error,.bb-form-control-hint{display:block;font-size:13px;margin-top:4px}.bb-form-control-error{color:#c23934}.bb-form-control-hint{color:#738694}"]
        })
    ], BbFormControl);
    return BbFormControl;
}(BbFormControlMixinBase));

var nextUniqueId$1 = 0;
var BbFilePickerBase = /** @class */ (function () {
    function BbFilePickerBase() {
    }
    return BbFilePickerBase;
}());
var BbFilePickerMixinBase = mixinRequired(mixinDisabled(mixinGrouped(mixinError(mixinHideErrors(BbFilePickerBase)))));
var BbFilePicker = /** @class */ (function (_super) {
    __extends(BbFilePicker, _super);
    function BbFilePicker() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // Inputs.
        _this.label = null;
        _this.hint = null;
        _this.accept = null;
        _this.showImages = true;
        _this.allowDragging = true;
        // State.
        _this.labelId = "bb-file-picker-" + nextUniqueId$1++;
        _this.hovered = false;
        _this.value$ = new BehaviorSubject({});
        _this.valueChange = new EventEmitter();
        _this.onTouchedCallback = function () {
        };
        _this.onChangeCallback = function () {
        };
        _this.saveFile = function (files) {
            // Validate the files exist.
            if (!files) {
                return;
            }
            // Retrieve the first file.
            var file = files[0];
            // Validate the first file exists.
            if (!file) {
                return;
            }
            // Save the file.
            _this.value = file;
        };
        _this.getFileListFromEvent = function (event) {
            var element = event.target;
            if (!element) {
                return null;
            }
            return element.files;
        };
        _this.preventDefault = function (event) {
            event.preventDefault();
            event.stopPropagation();
        };
        return _this;
    }
    BbFilePicker_1 = BbFilePicker;
    Object.defineProperty(BbFilePicker.prototype, "value", {
        get: function () {
            var file = this.value$.getValue().file;
            return file;
        },
        set: function (value) {
            this.value$.next({ file: value });
            this.valueChange.emit(value);
        },
        enumerable: true,
        configurable: true
    });
    BbFilePicker.prototype.onDrag = function (event, isHovered) {
        // Validate the input can support drag.
        if (!this.allowDragging || this.disabled) {
            return;
        }
        // Prevent the default event action
        // and start the hovering.
        this.preventDefault(event);
        this.hovered = isHovered;
    };
    BbFilePicker.prototype.onDrop = function (event) {
        // Validate the input can support drag.
        if (!this.allowDragging || this.disabled) {
            return;
        }
        // Prevent the default event action
        // and stop the hovering.
        this.preventDefault(event);
        this.hovered = false;
        // Save the file.
        this.saveFile(event['dataTransfer'].files);
        // Run the on change callback.
        this.onChangeCallback(this.value);
    };
    BbFilePicker.prototype.onButtonPressed = function (file) {
        if (file) {
            this.value = null;
            this.onChangeCallback(this.value);
            return;
        }
        return this.openFilePicker();
    };
    BbFilePicker.prototype.openFilePicker = function () {
        return this.fileInput
            && this.fileInput.nativeElement
            && this.fileInput.nativeElement.click
            && this.fileInput.nativeElement.click();
    };
    BbFilePicker.prototype.onFileChange = function (event) {
        // Validate the input is not disabled.
        if (this.disabled) {
            return;
        }
        // Extract the file from the event.
        var fileList = this.getFileListFromEvent(event);
        // Save the file.
        this.saveFile(fileList);
        // Run the on change callback.
        this.onChangeCallback(this.value);
        // Clear the file input.
        if (this.fileInput && this.fileInput.nativeElement) {
            this.fileInput.nativeElement.value = '';
        }
    };
    BbFilePicker.prototype.writeValue = function (value) {
        this.value = value;
    };
    BbFilePicker.prototype.registerOnChange = function (method) {
        this.onChangeCallback = method;
    };
    BbFilePicker.prototype.registerOnTouched = function (method) {
        this.onTouchedCallback = method;
    };
    BbFilePicker.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    BbFilePicker.prototype.onErrorChange = function (error) {
        this.error = !!error;
    };
    BbFilePicker.prototype.validate = function (_a) {
        var value = _a.value;
        var mimeTypeRegex = new RegExp((this.accept || '*').replace('*', '[^\\/,]+'));
        var isNotValid = !(value instanceof File) || !mimeTypeRegex.test(value === null || value === void 0 ? void 0 : value.type);
        return isNotValid && {
            invalidFileType: true
        };
    };
    var BbFilePicker_1;
    __decorate([
        ViewChild('fileInput', { static: true })
    ], BbFilePicker.prototype, "fileInput", void 0);
    __decorate([
        ContentChild('extra')
    ], BbFilePicker.prototype, "extraTemplate", void 0);
    __decorate([
        Input()
    ], BbFilePicker.prototype, "label", void 0);
    __decorate([
        Input()
    ], BbFilePicker.prototype, "hint", void 0);
    __decorate([
        Input()
    ], BbFilePicker.prototype, "accept", void 0);
    __decorate([
        Input()
    ], BbFilePicker.prototype, "showImages", void 0);
    __decorate([
        Input()
    ], BbFilePicker.prototype, "allowDragging", void 0);
    __decorate([
        Input()
    ], BbFilePicker.prototype, "value", null);
    __decorate([
        Output()
    ], BbFilePicker.prototype, "valueChange", void 0);
    __decorate([
        HostListener('dragover', ['$event', 'true']),
        HostListener('dragleave', ['$event', 'false'])
    ], BbFilePicker.prototype, "onDrag", null);
    __decorate([
        HostListener('drop', ['$event'])
    ], BbFilePicker.prototype, "onDrop", null);
    BbFilePicker = BbFilePicker_1 = __decorate([
        Component({
            selector: 'bb-file-picker',
            template: "<!-- The label of the input. -->\r\n<label *ngIf=\"label as labelContent\"\r\n       [for]=\"labelId\"\r\n       class=\"bb-file-picker-label\">\r\n    <ng-template [bbTemplate]=\"labelContent\">\r\n        {{ labelContent }}\r\n    </ng-template>\r\n</label>\r\n\r\n<!-- The input that can open the file picker. -->\r\n<input #fileInput\r\n       [accept]=\"accept\"\r\n       [disabled]=\"disabled\"\r\n       (change)=\"onFileChange($event)\"\r\n       class=\"bb-file-picker-input\"\r\n       type=\"file\"\r\n       tabindex=\"-1\">\r\n\r\n<div *ngIf=\"value$ | async as data\"\r\n     class=\"bb-file-picker-wrapper\">\r\n    <!-- The button that accepts files. -->\r\n    <div class=\"bb-file-picker-container\">\r\n        <button [class.destructive]=\"!!data?.file\"\r\n                [class.standalone]=\"!allowDragging\"\r\n                [disabled]=\"disabled\"\r\n                [id]=\"labelId\"\r\n                (click)=\"onButtonPressed(data?.file)\"\r\n                type=\"button\"\r\n                class=\"bb-file-picker-button\">\r\n            <ng-container *ngTemplateOutlet=\"data?.file ? closeIcon : uploadIcon\"></ng-container>\r\n            {{ (data?.file ? 'file-picker.remove' : 'file-picker.choose') | bbLocalize }}\r\n        </button>\r\n        <div *ngIf=\"allowDragging\"\r\n             class=\"bb-file-picker-zone\">\r\n            {{ 'file-picker.drop' | bbLocalize }}\r\n        </div>\r\n    </div>\r\n\r\n    <ng-container *ngIf=\"!!extraTemplate\">\r\n        <ng-container *ngTemplateOutlet=\"extraTemplate; context: {$implicit: data?.file}\"></ng-container>\r\n    </ng-container>\r\n\r\n    <ng-container *ngIf=\"showImages\">\r\n        <div *ngIf=\"data?.file | bbFileImage | async as image\"\r\n             class=\"bb-file-picker-image-container\">\r\n            <div class=\"bb-file-picker-image-wrapper\">\r\n                <div [@bbFilePickerImageAnimation]=\"true\"\r\n                     [style.padding-top.%]=\"image?.aspectRatio * 100\"\r\n                     [style.background-image]=\"image?.background\"\r\n                     class=\"bb-file-picker-image\">\r\n                </div>\r\n            </div>\r\n        </div>\r\n    </ng-container>\r\n\r\n    <!-- Extra information of the file. -->\r\n    <div [class.visible]=\"!!data?.file\"\r\n         class=\"bb-file-picker-info\">\r\n        <span *ngIf=\"data?.file?.name as name\">{{ name }}</span>\r\n        <span *ngIf=\"data?.file?.size as size\">{{ size | bbFileSize }}</span>\r\n    </div>\r\n</div>\r\n\r\n<bb-form-error *ngIf=\"!hideErrors\"\r\n               (errorChange)=\"onErrorChange($event)\">\r\n</bb-form-error>\r\n\r\n<!-- The file picker hint. -->\r\n<p *ngIf=\"hint as hintContent\"\r\n   class=\"bb-file-picker-hint\">\r\n    <ng-template [bbTemplate]=\"hintContent\">\r\n        {{ hintContent }}\r\n    </ng-template>\r\n</p>\r\n\r\n<!-- Upload icon. -->\r\n<ng-template #uploadIcon>\r\n    <svg xmlns=\"http://www.w3.org/2000/svg\"\r\n         class=\"bb-file-picker-icon\"\r\n         width=\"24\"\r\n         height=\"24\">\r\n        <path fill=\"currentColor\"\r\n              d=\"M2 12.5C2 9.46 4.46 7 7.5 7H18c2.21 0 4 1.79 4 4s-1.79 4-4 4H9.5a2.5 2.5 0 010-5H17v2H9.41c-.55 0-.55 1 0 1H18c1.1 0 2-.9 2-2s-.9-2-2-2H7.5C5.57 9 4 10.57 4 12.5S5.57 16 7.5 16H17v2H7.5C4.46 18 2 15.54 2 12.5z\"></path>\r\n        <path fill=\"none\" d=\"M0 0h24v24H0V0z\"></path>\r\n    </svg>\r\n</ng-template>\r\n\r\n<!-- Close icon. -->\r\n<ng-template #closeIcon>\r\n    <svg xmlns=\"http://www.w3.org/2000/svg\"\r\n         class=\"bb-file-picker-icon\"\r\n         width=\"24\"\r\n         height=\"24\">\r\n        <path fill=\"currentColor\"\r\n              d=\"M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z\"></path>\r\n        <path d=\"M0 0h24v24H0z\" fill=\"none\"></path>\r\n    </svg>\r\n</ng-template>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            providers: [
                {
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: forwardRef(function () { return BbFilePicker_1; }),
                    multi: true
                },
                {
                    provide: NG_VALIDATORS,
                    useExisting: BbFilePicker_1,
                    multi: true
                }
            ],
            host: {
                'class': 'bb-file-picker',
                '[class.required]': 'required',
                '[class.disabled]': 'disabled',
                '[class.grouped]': 'grouped',
                '[class.hovered]': 'hovered',
                '[class.error]': 'error'
            },
            animations: [
                trigger('bbFilePickerImageAnimation', [
                    transition(':enter', [
                        style({ opacity: 0 }),
                        animate('200ms', style({ opacity: 1 }))
                    ]),
                    transition(':leave', [
                        animate('200ms', style({ opacity: 0 }))
                    ])
                ])
            ],
            inputs: ['disabled', 'hideErrors', 'grouped', 'required'],
            preserveWhitespaces: false,
            styles: [".bb-file-picker{display:block}.bb-file-picker.disabled>.bb-file-picker-wrapper{cursor:default;background-color:rgba(204,204,204,.64)}.bb-file-picker.grouped{margin-bottom:20px}.bb-file-picker.full>.bb-file-picker-wrapper{width:100%}.bb-file-picker:not(.disabled) .bb-file-picker-button{background-color:#fff}.bb-file-picker:not(.disabled).hovered>.bb-file-picker-wrapper,.bb-file-picker:not(.disabled):focus>.bb-file-picker-wrapper{box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(0,123,255,.2)}.bb-file-picker.required>.bb-file-picker-label:after{content:\"*\";color:#c23934;font-size:12px;line-height:1.5}.bb-file-picker.error>.bb-file-picker-label{color:#c23934}.bb-file-picker.error>.bb-file-picker-hint{display:none}.bb-file-picker.error>.bb-file-picker-wrapper{border:1px solid #962b26;background-color:rgba(194,57,52,.25)!important;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(194,57,52,.4)!important}.bb-file-picker.error .bb-file-picker-button{background-color:transparent}.bb-file-picker-wrapper{max-width:100%;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;border-radius:3px;display:-ms-inline-flexbox;display:inline-flex;-ms-flex-direction:column;flex-direction:column;border:1px solid #b6bbc1;background-color:#fff8ff;transition:box-shadow .2s cubic-bezier(0,0,.2,1);box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1)}.bb-file-picker-label{color:#000;display:block;position:relative;margin-bottom:4px}.bb-file-picker-input{opacity:0;z-index:-1;width:.1px;height:.1px;overflow:hidden;position:absolute}.bb-file-picker-container{min-height:38px;display:-ms-inline-flexbox;display:inline-flex}.bb-file-picker-button{margin:0;z-index:1;border:none;color:#2196f3;line-height:1;cursor:pointer;padding:0 12px;-webkit-appearance:none;-moz-appearance:none;appearance:none;font-weight:500;-ms-flex-align:center;align-items:center;white-space:nowrap;display:-ms-inline-flexbox;display:inline-flex;border-top-left-radius:3px;border-bottom-left-radius:3px;border-right:1px solid #b6bbc1;background-color:rgba(255,255,255,.5);transition:box-shadow .2s cubic-bezier(0,0,.2,1)}.bb-file-picker-button:focus{box-shadow:0 0 0 3px rgba(0,123,255,.2)}.bb-file-picker-button:disabled{cursor:default}.bb-file-picker-button.destructive{color:#f55656}.bb-file-picker-button.standalone{width:100%;border-right:none;border-top-right-radius:3px;border-bottom-right-radius:3px}.bb-file-picker-icon{margin-right:4px}.bb-file-picker-zone{-ms-flex:1;flex:1;color:#848f99;font-size:14px;padding:8px 12px;line-height:18px;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;-ms-flex-pack:center;justify-content:center}.bb-file-picker-info{height:0;color:#848f99;padding:0 12px;overflow:hidden;line-height:36px;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;background-color:#fff8ff;border-top:0 solid #b6bbc1;transition:height .2s cubic-bezier(0,0,.2,1)}.bb-file-picker-info.visible{height:38px;border-top-width:1px}.bb-file-picker-info>span{opacity:.4;max-width:100%;font-size:14px;overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.bb-file-picker-info>span:first-child{-ms-flex:1;flex:1}.bb-file-picker-info>span:last-child{margin-left:12px}.bb-file-picker-image-container{overflow:hidden;border-top:1px solid #b6bbc1}.bb-file-picker-image-wrapper{width:100%;max-width:40%;margin:20px auto}.bb-file-picker-image{width:100%;display:-ms-flexbox;display:flex;overflow:hidden;position:relative;border-radius:4px;will-change:opacity;background-size:cover;background-position:center;background-repeat:no-repeat}.bb-file-picker-hint{display:block;color:#738694;font-size:13px;margin-top:4px}"]
        })
    ], BbFilePicker);
    return BbFilePicker;
}(BbFilePickerMixinBase));

var BbSuffix = /** @class */ (function () {
    function BbSuffix() {
    }
    BbSuffix = __decorate([
        Directive({
            selector: '[bbSuffix]',
            host: {
                'class': 'bb-suffix'
            }
        })
    ], BbSuffix);
    return BbSuffix;
}());
var BbPrefix = /** @class */ (function () {
    function BbPrefix() {
    }
    BbPrefix = __decorate([
        Directive({
            selector: '[bbPrefix]',
            host: {
                'class': 'bb-prefix'
            }
        })
    ], BbPrefix);
    return BbPrefix;
}());

var BbFormSubmit = /** @class */ (function () {
    function BbFormSubmit(_host, _formDirective) {
        this._host = _host;
        this._formDirective = _formDirective;
    }
    BbFormSubmit.prototype.ngOnInit = function () {
        // Submit changes.
        var regularSubmitChanges$ = fromEvent(this.nativeElement, 'submit').pipe(mapTo(true));
        // Reactive form submit changes.
        var reactiveSubmitChanges$ = this.reactiveSubmitChanges();
        // Reset changes.
        var resetChanges$ = fromEvent(this.nativeElement, 'reset').pipe(mapTo(false));
        // Listen to the submit status.
        this.submitStatus$ = merge(regularSubmitChanges$, reactiveSubmitChanges$, resetChanges$).pipe(startWith(false), distinctUntilChanged(), share());
    };
    BbFormSubmit.prototype.submit = function () {
        this._formDirective && this._formDirective.ngSubmit.emit();
    };
    BbFormSubmit.prototype.reset = function () {
        return this.nativeElement.reset();
    };
    Object.defineProperty(BbFormSubmit.prototype, "nativeElement", {
        get: function () {
            return this._host && this._host.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    BbFormSubmit.prototype.reactiveSubmitChanges = function () {
        if (!this._formDirective) {
            return EMPTY;
        }
        return this._formDirective.ngSubmit.pipe(mapTo(true));
    };
    BbFormSubmit.ctorParameters = function () { return [
        { type: ElementRef },
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Self }] }
    ]; };
    BbFormSubmit = __decorate([
        Directive({
            selector: 'form',
            exportAs: 'bbForm'
        }),
        __param(1, Optional()), __param(1, Self())
    ], BbFormSubmit);
    return BbFormSubmit;
}());

var BbSelect = /** @class */ (function () {
    function BbSelect(_elementRef, _ngControl, _formControl) {
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._formControl = _formControl;
        this.hasArrow = true;
        this._disabled = false;
        this._required = false;
    }
    BbSelect.prototype.ngOnInit = function () {
        // Set an arrow on the form control if
        // it requires one.
        this._formControl.arrow = this.hasArrow;
    };
    BbSelect.prototype.ngDoCheck = function () {
        if (this._ngControl) {
            this.disabled = this._ngControl.disabled;
        }
    };
    BbSelect.prototype.ngOnDestroy = function () {
        var _a;
        (_a = this._subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    };
    BbSelect.prototype.focus = function () {
        var _a, _b;
        return (_b = (_a = this._elementRef) === null || _a === void 0 ? void 0 : _a.nativeElement) === null || _b === void 0 ? void 0 : _b.focus();
    };
    BbSelect.prototype.setFocus = function (hasFocus) {
        if (!this._formControl) {
            return;
        }
        this._formControl.focused = hasFocus;
    };
    Object.defineProperty(BbSelect.prototype, "id", {
        get: function () {
            var _a, _b;
            return (_b = (_a = this._formControl) === null || _a === void 0 ? void 0 : _a.labelId) !== null && _b !== void 0 ? _b : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbSelect.prototype, "disabled", {
        get: function () {
            if (this._ngControl && this._ngControl.disabled !== null) {
                return this._ngControl.disabled;
            }
            return this._disabled;
        },
        set: function (newValue) {
            this._disabled = coerceBooleanProperty(newValue);
            if (!this._formControl) {
                return;
            }
            this._formControl.disabled = this._disabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbSelect.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (newValue) {
            this._required = coerceBooleanProperty(newValue);
            if (!this._formControl) {
                return;
            }
            this._formControl.required = this._required;
        },
        enumerable: true,
        configurable: true
    });
    BbSelect.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: BbFormControl, decorators: [{ type: Optional }, { type: Host }] }
    ]; };
    __decorate([
        HostBinding('attr.id')
    ], BbSelect.prototype, "id", null);
    __decorate([
        Input()
    ], BbSelect.prototype, "disabled", null);
    __decorate([
        Input()
    ], BbSelect.prototype, "required", null);
    BbSelect = __decorate([
        Directive({
            selector: 'select[bbInput]',
            host: {
                '[disabled]': 'disabled',
                '(focus)': 'setFocus(true)',
                '(blur)': 'setFocus(false)'
            }
        }),
        __param(1, Optional()), __param(1, Self()),
        __param(2, Optional()), __param(2, Host())
    ], BbSelect);
    return BbSelect;
}());
var BbInput = /** @class */ (function (_super) {
    __extends(BbInput, _super);
    function BbInput(_elementRef, _ngControl, _formControl) {
        var _this = _super.call(this, _elementRef, _ngControl, _formControl) || this;
        _this._elementRef = _elementRef;
        _this._ngControl = _ngControl;
        _this._formControl = _formControl;
        _this.hasArrow = false;
        _this._readonly = false;
        return _this;
    }
    Object.defineProperty(BbInput.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (newValue) {
            this._readonly = coerceBooleanProperty(newValue);
            if (!this._formControl) {
                return;
            }
            this._formControl.readonly = this._readonly;
        },
        enumerable: true,
        configurable: true
    });
    BbInput.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: BbFormControl, decorators: [{ type: Optional }, { type: Host }] }
    ]; };
    __decorate([
        Input()
    ], BbInput.prototype, "readonly", null);
    BbInput = __decorate([
        Directive({
            selector: 'input[bbInput], textarea[bbInput]',
            host: {
                '[disabled]': 'disabled',
                '[readonly]': 'readonly',
                '(focus)': 'setFocus(true)',
                '(blur)': 'setFocus(false)'
            }
        }),
        __param(1, Optional()), __param(1, Self()),
        __param(2, Optional()), __param(2, Host())
    ], BbInput);
    return BbInput;
}(BbSelect));

var BbFormGroupBase = /** @class */ (function () {
    function BbFormGroupBase() {
    }
    return BbFormGroupBase;
}());
var BbFormGroupMixinBase = mixinGrouped(BbFormGroupBase);
var BbFormGroup = /** @class */ (function (_super) {
    __extends(BbFormGroup, _super);
    function BbFormGroup() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BbFormGroup = __decorate([
        Component({
            selector: 'bb-form-group',
            template: "<ng-content></ng-content>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            host: {
                'class': 'bb-form-group',
                '[class.grouped]': 'grouped'
            },
            inputs: ['grouped'],
            preserveWhitespaces: false,
            styles: [".bb-form-group{width:100%;display:-ms-flexbox;display:flex;-ms-flex-wrap:wrap;flex-wrap:wrap;margin-top:-10px}.bb-form-group>*{margin-top:10px}.bb-form-group>:not(:first-child){margin-left:10px}.bb-form-group.start{-ms-flex-pack:start;justify-content:flex-start}.bb-form-group.end{-ms-flex-pack:end;justify-content:flex-end}.bb-form-group.vertical{-ms-flex-direction:column;flex-direction:column;-ms-flex-pack:normal;justify-content:normal}.bb-form-group.vertical>*{margin-left:0}.bb-form-group.vertical.start{-ms-flex-align:start;align-items:flex-start}.bb-form-group.vertical.end{-ms-flex-align:end;align-items:flex-end}.bb-form-group.grouped{margin-bottom:20px}"]
        })
    ], BbFormGroup);
    return BbFormGroup;
}(BbFormGroupMixinBase));

var BbFormError = /** @class */ (function () {
    function BbFormError(_form, _control, _errors) {
        this._form = _form;
        this._control = _control;
        this._errors = _errors;
        // Outputs.
        this.errorChange = new EventEmitter();
        // Subscriptions.
        this._ngControl = null;
    }
    Object.defineProperty(BbFormError.prototype, "control", {
        get: function () {
            return this._ngControl;
        },
        // Inputs.
        set: function (value) {
            this._ngControl = value;
        },
        enumerable: true,
        configurable: true
    });
    BbFormError.prototype.ngOnInit = function () {
        var _this = this;
        // Set the control if it was not set before.
        if (!this.control) {
            this.control = this._control;
        }
        // Listen for the submit changes
        var submitStatus$ = this.submitStatusChanges();
        // An observable that emits error changes.
        var errorChanges$ = this.statusChanges().pipe(map(function () { return _this.getError(_this.control.errors); }), distinctUntilChanged(), map(function (error) { return typeof error === 'string' ? { token: error } : error; }));
        // Whenever the user submitted the form we want to
        // listen for any error changes.
        this.error$ = combineLatest([submitStatus$, errorChanges$]).pipe(map(function (_a) {
            var _b = __read(_a, 2), submitted = _b[0], error = _b[1];
            // If not submitted return null.
            if (!submitted) {
                return null;
            }
            // Return the error if submitted.
            return error;
        }), tap(function (value) { return _this.errorChange.emit(value); }));
    };
    BbFormError.prototype.getError = function (errors) {
        // Validate the errors exist.
        if (!errors) {
            return null;
        }
        // Get the first error in the errors.
        var _a = __read(Object.keys(errors), 1), errorName = _a[0];
        var error = errors[errorName];
        // Validate the error exists.
        if (!error) {
            return null;
        }
        // Map the arrays into a object so we can
        // access the functions by a key-lookup.
        var errorFunctions = this._errors.reduce(function (previous, current) { return (__assign(__assign({}, previous), current)); }, {});
        // Get the error function.
        var errorFunction = errorFunctions[errorName] || errorFunctions['unknown'];
        // Validate the error function exists.
        if (!errorFunction) {
            return null;
        }
        // Return the parsed error.
        return errorFunction(error);
    };
    BbFormError.prototype.submitStatusChanges = function () {
        if (!this._form) {
            return of(true);
        }
        return this._form.submitStatus$;
    };
    BbFormError.prototype.statusChanges = function () {
        if (!this.control) {
            return EMPTY;
        }
        if (!this.control.statusChanges) {
            return EMPTY;
        }
        return this.control.statusChanges.pipe(startWith(this.control.status), delay(0));
    };
    BbFormError.ctorParameters = function () { return [
        { type: BbFormSubmit, decorators: [{ type: Optional }] },
        { type: NgControl, decorators: [{ type: Optional }] },
        { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [ELEMENTS_ERRORS,] }] }
    ]; };
    __decorate([
        Input()
    ], BbFormError.prototype, "control", null);
    __decorate([
        Output()
    ], BbFormError.prototype, "errorChange", void 0);
    BbFormError = __decorate([
        Component({
            selector: 'bb-form-error',
            template: "<ng-container *ngIf=\"error$ | async as error\">\r\n    <p *ngIf=\"error?.token | bbLocalize:{optional: true, data: error?.data} as message\"\r\n       class=\"bb-form-error-message\">\r\n        {{ message }}\r\n    </p>\r\n</ng-container>",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            host: {
                'class': 'bb-form-error'
            },
            preserveWhitespaces: false,
            styles: [".bb-form-error{display:block}.bb-form-error-message{color:#c23934;display:block;font-size:13px;margin-top:4px}"]
        }),
        __param(0, Optional()),
        __param(1, Optional()),
        __param(2, Optional()), __param(2, Inject(ELEMENTS_ERRORS))
    ], BbFormError);
    return BbFormError;
}());

var BbFocusTrap = /** @class */ (function () {
    function BbFocusTrap(_platform, _elementRef) {
        this._platform = _platform;
        this._elementRef = _elementRef;
        this._focusableElements = [
            'a[href]',
            'area[href]',
            'input:not([disabled]):not([type="hidden"]):not([aria-hidden])',
            'select:not([disabled]):not([aria-hidden])',
            'textarea:not([disabled]):not([aria-hidden])',
            'button:not([disabled]):not([aria-hidden])',
            'iframe',
            'object',
            'embed',
            '[contenteditable]',
            '[tabindex]:not([tabindex^="-"])'
        ];
        this.focus = function (element) {
            return element && element.focus && element.focus();
        };
        this.isTabEvent = function (event) {
            return event.key === 'Tab' || event.keyCode === 9;
        };
    }
    BbFocusTrap.prototype.onKeydown = function (event) {
        // Validate it is a tab event.
        if (!this.isTabEvent(event)) {
            return;
        }
        // Trap the focus inside the element.
        return this.trapFocus(event);
    };
    BbFocusTrap.prototype.trapFocus = function (event) {
        // Validate that the DOM is available.
        if (!this._platform.isBrowser) {
            return;
        }
        // Get all focusable nodes.
        var focusableNodes = this.getFocusableNodes();
        // Focus the first available element if the focus
        // is not in the modal.
        if (!this.element.contains(document.activeElement)) {
            return this.focus(focusableNodes[0]);
        }
        var focusedItemIndex = focusableNodes.indexOf(document.activeElement);
        if (event.shiftKey && focusedItemIndex === 0) {
            this.focus(focusableNodes[focusableNodes.length - 1]);
            return event.preventDefault();
        }
        if (!event.shiftKey && focusedItemIndex === focusableNodes.length - 1) {
            this.focus(focusableNodes[0]);
            return event.preventDefault();
        }
    };
    Object.defineProperty(BbFocusTrap.prototype, "element", {
        get: function () {
            return this._elementRef.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    BbFocusTrap.prototype.getFocusableNodes = function () {
        var nodes = this.element.querySelectorAll(this._focusableElements);
        return Array.apply(void 0, __spread(nodes));
    };
    BbFocusTrap.ctorParameters = function () { return [
        { type: Platform },
        { type: ElementRef }
    ]; };
    __decorate([
        HostListener('keydown', ['$event'])
    ], BbFocusTrap.prototype, "onKeydown", null);
    BbFocusTrap = __decorate([
        Directive({
            selector: '[bbFocusTrap]'
        })
    ], BbFocusTrap);
    return BbFocusTrap;
}());

var BbButtonBase = /** @class */ (function () {
    function BbButtonBase(_elementRef) {
        this._elementRef = _elementRef;
    }
    return BbButtonBase;
}());
var BbButtonMixinBase = mixinDisabled(mixinLoad(BbButtonBase));
var BbButton = /** @class */ (function (_super) {
    __extends(BbButton, _super);
    function BbButton(_elementRef) {
        return _super.call(this, _elementRef) || this;
    }
    Object.defineProperty(BbButton.prototype, "nativeElement", {
        get: function () {
            return this._elementRef.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    BbButton.prototype.focus = function () {
        this.nativeElement && this.nativeElement.focus();
    };
    BbButton.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    BbButton = __decorate([
        Component({
            selector: 'button[bb-button]',
            template: "<bb-spinner [class.visible]=\"loading\"\r\n            class=\"bb-button-spinner small\">\r\n</bb-spinner>\r\n\r\n<ng-content></ng-content>\r\n",
            exportAs: 'bbButton',
            host: {
                'class': 'bb-button',
                '[class.loading]': 'loading',
                '[attr.disabled]': 'disabled || loading || null'
            },
            inputs: ['disabled', 'loading'],
            encapsulation: ViewEncapsulation.None,
            changeDetection: ChangeDetectionStrategy.OnPush,
            styles: [".bb-button{height:38px;padding:0 8px;cursor:pointer;font-size:16px;font-weight:500;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;position:relative;font-style:normal;border-radius:4px;text-align:center;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;text-decoration:none;-ms-flex-pack:center;justify-content:center;background-color:#fff;color:#2d3c4d!important;border:1px solid #d4d8e1;transition-duration:.2s;box-shadow:0 6px 6px -6px rgba(0,0,0,.1);transition-timing-function:cubic-bezier(0,0,.2,1);transition-property:padding-left,background-color,box-shadow}.bb-button:hover{background-color:#fcfcfb}.bb-button:active,.bb-button:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(220,224,233,.5)}.bb-button.full{width:100%}.bb-button.full.loading{padding-left:42px}.bb-button.primary{border-color:#2f2b85;color:#fff!important;background-color:#5b53ff}.bb-button.primary:hover{background-color:#564ee6}.bb-button.primary:active,.bb-button.primary:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(91,83,255,.5)}.bb-button.destructive{border-color:#8f2020;color:#fff!important;background-color:#d32f2f}.bb-button.destructive:hover{background-color:#c32929}.bb-button.destructive:active,.bb-button.destructive:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(211,47,47,.5)}.bb-button.loading,.bb-button.loading:active,.bb-button.loading:focus,.bb-button.loading:hover,.bb-button:disabled,.bb-button:disabled:active,.bb-button:disabled:focus,.bb-button:disabled:hover,.bb-button[disabled],.bb-button[disabled]:active,.bb-button[disabled]:focus,.bb-button[disabled]:hover{opacity:.4;cursor:default;box-shadow:none}.bb-button-spinner{width:0;opacity:0;overflow:hidden;visibility:hidden;padding:0!important;transition:width .2s cubic-bezier(0,0,.2,1),opacity .1s cubic-bezier(0,0,.2,1)}.bb-button-spinner.visible{opacity:1;width:20px;margin-right:8px;visibility:visible}.bb-button.loading.full>.bb-button-spinner{left:8px;margin-right:0;position:absolute}"]
        })
    ], BbButton);
    return BbButton;
}(BbButtonMixinBase));
var BbAnchor = /** @class */ (function (_super) {
    __extends(BbAnchor, _super);
    function BbAnchor(_elementRef) {
        return _super.call(this, _elementRef) || this;
    }
    BbAnchor.prototype.stopDisabledEvents = function (event) {
        // If the button is not disabled (and not loading) let all events pass.
        if (!this.disabled && !this.loading) {
            return;
        }
        // If the button is disabled and/or loading stop
        // the event from bubbling up.
        event.preventDefault();
        event.stopImmediatePropagation();
    };
    BbAnchor.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Input()
    ], BbAnchor.prototype, "tabIndex", void 0);
    BbAnchor = __decorate([
        Component({
            selector: 'a[bb-button]',
            template: "<bb-spinner [class.visible]=\"loading\"\r\n            class=\"bb-button-spinner small\">\r\n</bb-spinner>\r\n\r\n<ng-content></ng-content>\r\n",
            exportAs: 'bbButton, bbAnchor',
            host: {
                'class': 'bb-button',
                '[attr.tabindex]': '(disabled || loading) ? -1 : (tabIndex || 0)',
                '[attr.disabled]': 'disabled || loading || null',
                '[attr.aria-disabled]': '(disabled || loading).toString()',
                '(click)': 'stopDisabledEvents($event)'
            },
            inputs: ['disabled', 'loading'],
            encapsulation: ViewEncapsulation.None,
            changeDetection: ChangeDetectionStrategy.OnPush,
            styles: [".bb-button{height:38px;padding:0 8px;cursor:pointer;font-size:16px;font-weight:500;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;position:relative;font-style:normal;border-radius:4px;text-align:center;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;text-decoration:none;-ms-flex-pack:center;justify-content:center;background-color:#fff;color:#2d3c4d!important;border:1px solid #d4d8e1;transition-duration:.2s;box-shadow:0 6px 6px -6px rgba(0,0,0,.1);transition-timing-function:cubic-bezier(0,0,.2,1);transition-property:padding-left,background-color,box-shadow}.bb-button:hover{background-color:#fcfcfb}.bb-button:active,.bb-button:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(220,224,233,.5)}.bb-button.full{width:100%}.bb-button.full.loading{padding-left:42px}.bb-button.primary{border-color:#2f2b85;color:#fff!important;background-color:#5b53ff}.bb-button.primary:hover{background-color:#564ee6}.bb-button.primary:active,.bb-button.primary:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(91,83,255,.5)}.bb-button.destructive{border-color:#8f2020;color:#fff!important;background-color:#d32f2f}.bb-button.destructive:hover{background-color:#c32929}.bb-button.destructive:active,.bb-button.destructive:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(211,47,47,.5)}.bb-button.loading,.bb-button.loading:active,.bb-button.loading:focus,.bb-button.loading:hover,.bb-button:disabled,.bb-button:disabled:active,.bb-button:disabled:focus,.bb-button:disabled:hover,.bb-button[disabled],.bb-button[disabled]:active,.bb-button[disabled]:focus,.bb-button[disabled]:hover{opacity:.4;cursor:default;box-shadow:none}.bb-button-spinner{width:0;opacity:0;overflow:hidden;visibility:hidden;padding:0!important;transition:width .2s cubic-bezier(0,0,.2,1),opacity .1s cubic-bezier(0,0,.2,1)}.bb-button-spinner.visible{opacity:1;width:20px;margin-right:8px;visibility:visible}.bb-button.loading.full>.bb-button-spinner{left:8px;margin-right:0;position:absolute}"]
        })
    ], BbAnchor);
    return BbAnchor;
}(BbButton));

var BbTemplate = /** @class */ (function () {
    function BbTemplate(_templateRef, _viewContainerRef) {
        this._templateRef = _templateRef;
        this._viewContainerRef = _viewContainerRef;
    }
    Object.defineProperty(BbTemplate.prototype, "bbTemplate", {
        set: function (content) {
            // Get the template.
            var template = content instanceof TemplateRef
                ? content
                : this._templateRef;
            // Clear the view container ref and create the view.
            this._viewContainerRef.clear();
            this._viewContainerRef.createEmbeddedView(template);
        },
        enumerable: true,
        configurable: true
    });
    BbTemplate.ctorParameters = function () { return [
        { type: TemplateRef },
        { type: ViewContainerRef }
    ]; };
    __decorate([
        Input()
    ], BbTemplate.prototype, "bbTemplate", null);
    BbTemplate = __decorate([
        Directive({
            selector: '[bbTemplate]'
        })
    ], BbTemplate);
    return BbTemplate;
}());

var BbAutosize = /** @class */ (function () {
    function BbAutosize(_renderer, _elementRef) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        // Min/max heights for the textarea.
        this.minHeight = null;
        this.maxHeight = null;
        this.rows = 1;
    }
    Object.defineProperty(BbAutosize.prototype, "element", {
        get: function () {
            var _a;
            return (_a = this._elementRef) === null || _a === void 0 ? void 0 : _a.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    BbAutosize.prototype.ngAfterViewInit = function () {
        // Update the styles after the DOM has loaded.
        this.updateStyles();
    };
    BbAutosize.prototype.onWindowResize = function () {
        // Update the styles when the window is resized.
        this.updateStyles();
    };
    BbAutosize.prototype.onInputReceived = function () {
        // Update the styles after the textarea received input.
        this.updateStyles();
    };
    BbAutosize.prototype.updateStyles = function () {
        var _a, _b, _c;
        // Validate the element exists.
        if (!this.element) {
            return;
        }
        // Calculate border height which is not included in the scroll height.
        var borderHeight = ((_a = this.element) === null || _a === void 0 ? void 0 : _a.offsetHeight) - ((_b = this.element) === null || _b === void 0 ? void 0 : _b.clientHeight);
        // Reset textarea height to auto that correctly calculate the new height.
        this.setHeight('auto');
        // Set new height.
        this.setHeight(((_c = this.element) === null || _c === void 0 ? void 0 : _c.scrollHeight) + borderHeight + "px");
    };
    BbAutosize.prototype.setHeight = function (value) {
        this._renderer.setStyle(this.element, 'height', value);
    };
    BbAutosize.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    __decorate([
        Input(), HostBinding('style.min-height')
    ], BbAutosize.prototype, "minHeight", void 0);
    __decorate([
        Input(), HostBinding('style.max-height')
    ], BbAutosize.prototype, "maxHeight", void 0);
    __decorate([
        Input(), HostBinding('rows')
    ], BbAutosize.prototype, "rows", void 0);
    __decorate([
        HostListener('window:resize')
    ], BbAutosize.prototype, "onWindowResize", null);
    __decorate([
        HostListener('input')
    ], BbAutosize.prototype, "onInputReceived", null);
    BbAutosize = __decorate([
        Directive({
            selector: 'textarea[bbAutosize]'
        })
    ], BbAutosize);
    return BbAutosize;
}());

var BbRelativeTime = /** @class */ (function () {
    function BbRelativeTime(_localize, _localeId) {
        this._localize = _localize;
        this._localeId = _localeId;
        this.isValidDate = function (date) {
            return typeof date === 'object' &&
                typeof date.getTime === 'function' &&
                !isNaN(date.getTime());
        };
        this.parseAsDate = function (value) {
            // Add a Z after the date so
            // it is treated as a UTC date.
            if (typeof value === 'string' && /^[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}$/.test(value)) {
                return new Date(value + "Z");
            }
            // Return the regular parsed date.
            return new Date(value);
        };
        this.isToday = function (date) {
            var today = new Date();
            return (date === null || date === void 0 ? void 0 : date.getDate()) === (today === null || today === void 0 ? void 0 : today.getDate()) &&
                (date === null || date === void 0 ? void 0 : date.getMonth()) === (today === null || today === void 0 ? void 0 : today.getMonth()) &&
                (date === null || date === void 0 ? void 0 : date.getFullYear()) === (today === null || today === void 0 ? void 0 : today.getFullYear());
        };
        this.isYesterday = function (date) {
            var _a, _b;
            var yesterday = new Date((_a = new Date()) === null || _a === void 0 ? void 0 : _a.setDate(((_b = new Date()) === null || _b === void 0 ? void 0 : _b.getDate()) - 1));
            return (date === null || date === void 0 ? void 0 : date.getDate()) === (yesterday === null || yesterday === void 0 ? void 0 : yesterday.getDate()) &&
                (date === null || date === void 0 ? void 0 : date.getMonth()) === (yesterday === null || yesterday === void 0 ? void 0 : yesterday.getMonth()) &&
                (date === null || date === void 0 ? void 0 : date.getFullYear()) === (yesterday === null || yesterday === void 0 ? void 0 : yesterday.getFullYear());
        };
        this.isTomorrow = function (date) {
            var _a, _b;
            var tomorrow = new Date((_a = new Date()) === null || _a === void 0 ? void 0 : _a.setDate(((_b = new Date()) === null || _b === void 0 ? void 0 : _b.getDate()) + 1));
            return (date === null || date === void 0 ? void 0 : date.getDate()) === (tomorrow === null || tomorrow === void 0 ? void 0 : tomorrow.getDate()) &&
                (date === null || date === void 0 ? void 0 : date.getMonth()) === (tomorrow === null || tomorrow === void 0 ? void 0 : tomorrow.getMonth()) &&
                (date === null || date === void 0 ? void 0 : date.getFullYear()) === (tomorrow === null || tomorrow === void 0 ? void 0 : tomorrow.getFullYear());
        };
        this._datePipe = new DatePipe(this._localeId);
    }
    BbRelativeTime.prototype.transform = function (value) {
        // Parse the value to a date object.
        var date = this.parseAsDate(value);
        // Check if the date is valid.
        if (!this.isValidDate(date)) {
            return value;
        }
        // Return the formatted date.
        return this.format(date);
    };
    BbRelativeTime.prototype.format = function (date) {
        // It happens tomorrow.
        if (this.isTomorrow(date)) {
            return this.formatString(date, 'times.tomorrow_at', 'shortTime');
        }
        // It happened today.
        if (this.isToday(date)) {
            return this.formatString(date, 'times.today_at', 'shortTime');
        }
        // It happened yesterday.
        if (this.isYesterday(date)) {
            return this.formatString(date, 'times.yesterday_at', 'shortTime');
        }
        // Just return the date.
        return this.formatString(date);
    };
    BbRelativeTime.prototype.formatString = function (date, token, format) {
        // Just return the parsed date.
        if (!this._localize || !token || !format) {
            return this._datePipe.transform(date, 'mediumDate');
        }
        // Return the translated formatting.
        return this._localize.translate({
            token: token,
            data: { date: this._datePipe.transform(date, format) }
        });
    };
    BbRelativeTime.ctorParameters = function () { return [
        { type: Localize, decorators: [{ type: Optional }] },
        { type: String, decorators: [{ type: Inject, args: [LOCALE_ID,] }] }
    ]; };
    BbRelativeTime = __decorate([
        Pipe({
            name: 'bbRelativeTime'
        }),
        __param(0, Optional()),
        __param(1, Inject(LOCALE_ID))
    ], BbRelativeTime);
    return BbRelativeTime;
}());

var nextUniqueId$2 = 0;
var BbCheckboxBase = /** @class */ (function () {
    function BbCheckboxBase() {
    }
    return BbCheckboxBase;
}());
var BbCheckboxMixinBase = mixinDisabled(mixinGrouped(BbCheckboxBase));
var BbCheckbox = /** @class */ (function (_super) {
    __extends(BbCheckbox, _super);
    function BbCheckbox(_changeDetection) {
        var _this = _super.call(this) || this;
        _this._changeDetection = _changeDetection;
        // Data.
        _this.labelId = "bb-checkbox-" + nextUniqueId$2++;
        _this._checked = false;
        // Methods.
        _this.onChange = function () {
        };
        _this.onTouched = function () {
        };
        return _this;
    }
    BbCheckbox_1 = BbCheckbox;
    Object.defineProperty(BbCheckbox.prototype, "checked", {
        get: function () {
            return this._checked;
        },
        set: function (value) {
            this._checked = coerceBooleanProperty(value);
            this._changeDetection.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    BbCheckbox.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    BbCheckbox.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    BbCheckbox.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this._changeDetection.markForCheck();
    };
    BbCheckbox.prototype.writeValue = function (newValue) {
        this.checked = newValue;
    };
    BbCheckbox.prototype.onInteractionEvent = function (event) {
        // Stop the propagation to prevent it from bubbling up.
        event.stopPropagation();
    };
    BbCheckbox.prototype.onInputClick = function (event) {
        // Stop the propagation to prevent it from bubbling up.
        event.stopPropagation();
        // Toggle the value.
        this.checked = !this.checked;
        // Emit the change event.
        this.onChange && this.onChange(this.checked);
    };
    var BbCheckbox_1;
    BbCheckbox.ctorParameters = function () { return [
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        Input()
    ], BbCheckbox.prototype, "tabIndex", void 0);
    __decorate([
        Input()
    ], BbCheckbox.prototype, "checked", null);
    BbCheckbox = BbCheckbox_1 = __decorate([
        Component({
            selector: 'bb-checkbox',
            template: "<input [id]=\"labelId\"\r\n       [checked]=\"checked\"\r\n       [disabled]=\"disabled\"\r\n       (change)=\"onInteractionEvent($event)\"\r\n       (click)=\"onInputClick($event)\"\r\n       class=\"bb-checkbox-input\"\r\n       tabindex=\"-1\"\r\n       type=\"checkbox\">\r\n\r\n<label [for]=\"labelId\"\r\n       [attr.tabindex]=\"disabled ? -1 : (tabIndex || 0)\"\r\n       (keyup.space)=\"onInputClick($event)\"\r\n       class=\"bb-checkbox-label\">\r\n    <span>\r\n        <svg width=\"12px\"\r\n             height=\"10px\"\r\n             viewbox=\"0 0 12 10\">\r\n            <polyline points=\"1.5 6 4.5 9 10.5 1\"></polyline>\r\n        </svg>\r\n    </span>\r\n    <span>\r\n        <ng-content></ng-content>\r\n    </span>\r\n</label>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            providers: [
                { provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return BbCheckbox_1; }), multi: true }
            ],
            host: {
                'class': 'bb-checkbox',
                '[class.grouped]': 'grouped'
            },
            inputs: ['grouped', 'disabled'],
            preserveWhitespaces: false,
            styles: [".bb-checkbox{display:block}.bb-checkbox+.bb-checkbox{margin-top:5px}.bb-checkbox.grouped{margin-bottom:20px}.bb-checkbox-input{display:none}.bb-checkbox-label{margin:0;display:-ms-flexbox;display:flex;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.bb-checkbox-label>span{display:inline-block;vertical-align:middle;transform:translate3d(0,0,0)}.bb-checkbox-label>span:first-child{width:18px;height:18px;min-width:18px;min-height:18px;position:relative;border-radius:3px;vertical-align:middle;border:1px solid #bdc4c9;transition:.2s}.bb-checkbox-label>span:last-child{padding-left:8px}.bb-checkbox-label>span:first-child>svg{top:3px;left:2px;fill:none;stroke:#fff;stroke-width:2;position:absolute;stroke-linecap:round;stroke-linejoin:round;stroke-dasharray:16px;stroke-dashoffset:16px;transition:.3s;transform:translate3d(0,0,0)}.bb-checkbox-input:not(:disabled):not(:checked)+.bb-checkbox-label:focus>span:first-child{box-shadow:0 0 0 3px rgba(222,205,205,.5)}.bb-checkbox-input:not(:disabled):checked+.bb-checkbox-label:focus>span:first-child{box-shadow:0 0 0 3px rgba(91,83,255,.5)}.bb-checkbox-input:checked+.bb-checkbox-label>span:first-child{border-color:#5b53ff;background-color:#5b53ff}.bb-checkbox-input:checked+.bb-checkbox-label>span:first-child>svg{stroke-dashoffset:0}.bb-checkbox-input:disabled+.bb-checkbox-label{cursor:default;pointer-events:none}.bb-checkbox-input:disabled:checked+.bb-checkbox-label>span:first-child{border-color:rgba(204,204,204,.64);background-color:rgba(204,204,204,.64)}"]
        })
    ], BbCheckbox);
    return BbCheckbox;
}(BbCheckboxMixinBase));

var BbSpinnerBase = /** @class */ (function () {
    function BbSpinnerBase() {
    }
    return BbSpinnerBase;
}());
var BbSpinnerMixinBase = mixinDisabled(BbSpinnerBase);
var BbSpinner = /** @class */ (function (_super) {
    __extends(BbSpinner, _super);
    function BbSpinner() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // Inputs.
        _this.color = '#303f9f';
        _this.alt = 'Loading...';
        return _this;
    }
    __decorate([
        Input()
    ], BbSpinner.prototype, "color", void 0);
    __decorate([
        Input()
    ], BbSpinner.prototype, "alt", void 0);
    BbSpinner = __decorate([
        Component({
            selector: 'bb-spinner',
            template: "<!--\r\n    The spinner element.\r\n\r\n    This element contains a circle with a 25% bar that spins\r\n    indefinitely. The border color can be set via the \"color\"\r\n    property.\r\n-->\r\n<div class=\"bb-spinner-container\">\r\n    <div [style.border-left-color]=\"color\"\r\n         [class.disabled]=\"disabled\"\r\n         class=\"bb-spinner-loader\"\r\n         aria-live=\"assertive\"\r\n         role=\"alert\">\r\n        {{ alt }}\r\n    </div>\r\n    <ng-content select=\"img\"></ng-content>\r\n</div>\r\n\r\n<!--\r\n    The content of the spinner.\r\n\r\n    This element contains the content of the spinner. The user\r\n    can set the content inside the tag.\r\n-->\r\n<span class=\"bb-spinner-content\">\r\n    <ng-content></ng-content>\r\n</span>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            host: { 'class': 'bb-spinner' },
            inputs: ['color', 'alt', 'disabled'],
            preserveWhitespaces: false,
            styles: [".bb-spinner{color:#565656;padding:20px 0;text-align:center;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;-ms-flex-direction:column;flex-direction:column;-ms-flex-pack:center;justify-content:center}.bb-spinner.block{display:-ms-flexbox;display:flex}.bb-spinner.inverse .bb-spinner-loader{border-color:rgba(255,255,255,.15)}.bb-spinner.inverse .bb-spinner-content{color:rgba(255,255,255,.5)}.bb-spinner.small .bb-spinner-loader,.bb-spinner.small .bb-spinner-loader:after{width:20px;height:20px;min-width:20px;min-height:20px}.bb-spinner.small .bb-spinner-loader{border-width:2px}.bb-spinner.small .bb-spinner-content{font-size:12px}.bb-spinner.medium .bb-spinner-loader,.bb-spinner.medium .bb-spinner-loader:after{width:60px;height:60px;min-width:60px;min-height:60px}.bb-spinner.medium .bb-spinner-loader{border-width:4px}.bb-spinner.medium .bb-spinner-content{font-size:16px}.bb-spinner.large .bb-spinner-loader,.bb-spinner.large .bb-spinner-loader:after{width:100px;height:100px;min-width:100px;min-height:100px}.bb-spinner.large .bb-spinner-loader{border-width:6px}.bb-spinner.large .bb-spinner-content{font-size:20px}.bb-spinner.horizontal{text-align:left;-ms-flex-direction:row;flex-direction:row}.bb-spinner.horizontal .bb-spinner-content:not(:empty){margin-left:10px;margin-top:0}.bb-spinner.vertical{text-align:center;-ms-flex-direction:column;flex-direction:column}.bb-spinner.vertical .bb-spinner-content:not(:empty){margin-left:0;margin-top:10px}.bb-spinner-container{display:-ms-flexbox;display:flex;position:relative}.bb-spinner-container>img{top:20%;left:20%;width:60%;height:60%;position:absolute}.bb-spinner-loader{margin:0;padding:0;overflow:hidden;text-indent:100%;color:transparent;position:relative;display:inline-block;vertical-align:middle;border:4px solid #f2f4f6;-webkit-animation:1s linear infinite spin;animation:1s linear infinite spin}.bb-spinner-loader,.bb-spinner-loader:after{width:60px;height:60px;min-width:60px;min-height:60px;border-radius:50%}.bb-spinner-loader.disabled{cursor:default;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;pointer-events:none;-webkit-animation-play-state:paused;animation-play-state:paused}.bb-spinner-content:not(:empty){font-size:16px;margin-top:10px;font-weight:500}@-webkit-keyframes spin{from{transform:rotate(0)}to{transform:rotate(360deg)}}@keyframes spin{from{transform:rotate(0)}to{transform:rotate(360deg)}}"]
        })
    ], BbSpinner);
    return BbSpinner;
}(BbSpinnerMixinBase));

var BbFocus = /** @class */ (function () {
    function BbFocus(_zone, _platform, _elementRef) {
        this._zone = _zone;
        this._platform = _platform;
        this._elementRef = _elementRef;
    }
    Object.defineProperty(BbFocus.prototype, "nativeElement", {
        get: function () {
            return this._elementRef.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    BbFocus.prototype.ngAfterViewInit = function () {
        var _this = this;
        // Run the method outside the Angular zone.
        this._zone.runOutsideAngular(function () { return _this.focus(); });
    };
    BbFocus.prototype.focus = function () {
        var _this = this;
        // Check if set timeout exists and the user is
        // using the site on desktop devices.
        if (!this._platform.isDesktop || !setTimeout) {
            return;
        }
        // Check if the element and the focus method exist, if so focus the element.
        if (!this.nativeElement || !this.nativeElement.focus) {
            return;
        }
        // Execute the focus method in a timeout.
        setTimeout(function () { return _this.nativeElement.focus(); }, 0);
    };
    BbFocus.ctorParameters = function () { return [
        { type: NgZone },
        { type: Platform },
        { type: ElementRef }
    ]; };
    BbFocus = __decorate([
        Directive({
            selector: '[bbFocus]'
        })
    ], BbFocus);
    return BbFocus;
}());

var BbFileImage = /** @class */ (function () {
    function BbFileImage(_image, _sanitizer) {
        this._image = _image;
        this._sanitizer = _sanitizer;
    }
    BbFileImage.prototype.transform = function (file) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                // If the file is not a file return.
                if (!(file instanceof File)) {
                    return [2 /*return*/, null];
                }
                return [2 /*return*/, this._image.fileToImage(file, [
                        'image/png',
                        'image/jpeg',
                        'image/gif',
                        'image/svg+xml'
                    ]).then(function (image) {
                        if (!image) {
                            return null;
                        }
                        return {
                            background: _this._sanitizer.bypassSecurityTrustStyle("url(" + image.src + ")"),
                            aspectRatio: image.height / image.width
                        };
                    }).catch(function () {
                        return null;
                    })];
            });
        });
    };
    BbFileImage.ctorParameters = function () { return [
        { type: ImageConverter },
        { type: DomSanitizer }
    ]; };
    BbFileImage = __decorate([
        Pipe({
            name: 'bbFileImage'
        })
    ], BbFileImage);
    return BbFileImage;
}());

var BbAvatar = /** @class */ (function () {
    function BbAvatar() {
        var _this = this;
        // Inputs.
        this.size = 40;
        // State.
        this.state$ = new BehaviorSubject('initials');
        // Data.
        this._src = null;
        this._title = null;
        this._initials = '!';
        this._color = 'rgb(255, 255, 255)';
        this._backgroundColor = 'rgb(66, 66, 66)';
        this.toRGb = function (value) {
            var baseRed = 128;
            var baseGreen = 128;
            var baseBlue = 128;
            var seed = value.charCodeAt(0) ^ value.charCodeAt(1);
            var rand_1 = Math.abs((Math.sin(seed++) * 10000)) % 256;
            var rand_2 = Math.abs((Math.sin(seed++) * 10000)) % 256;
            var rand_3 = Math.abs((Math.sin(seed++) * 10000)) % 256;
            var r = Math.round((rand_1 + baseRed) / 2);
            var g = Math.round((rand_2 + baseGreen) / 2);
            var b = Math.round((rand_3 + baseBlue) / 2);
            return { r: r, g: g, b: b };
        };
        this.getInitials = function (name) {
            // Get all the initials.
            var names = (name || '').split(' ');
            var initials = names.map(function (name) { return name.charAt(0).toUpperCase(); });
            // Check if there are any initials.
            if (initials.length <= 0) {
                return null;
            }
            // Get the first and last initials.
            if (initials.length > 1) {
                return "" + initials[0] + initials[initials.length - 1];
            }
            // Get only the first initial.
            return initials[0];
        };
        this.parseColorFormat = function (value) {
            // Hex 3 values.
            var match = value.match(/^#([0-9a-f]{3})$/i);
            if (match && match[1]) {
                var r = parseInt(match[1].charAt(0), 16) * 0x11;
                var g = parseInt(match[1].charAt(1), 16) * 0x11;
                var b = parseInt(match[1].charAt(2), 16) * 0x11;
                return { r: r, g: g, b: b };
            }
            // Hex 6 values.
            match = value.match(/^#([0-9a-f]{6})$/i);
            if (match && match[1]) {
                var r = parseInt(match[1].substr(0, 2), 16);
                var g = parseInt(match[1].substr(2, 2), 16);
                var b = parseInt(match[1].substr(4, 2), 16);
                return { r: r, g: g, b: b };
            }
            // Rgb values.
            match = value.match(/^rgb\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)$/i);
            if (match) {
                var r = parseInt(match[1], 10);
                var g = parseInt(match[2], 10);
                var b = parseInt(match[3], 10);
                return { r: r, g: g, b: b };
            }
            // Not a supported color format.
            _this.logWarning("BbAvatar: Unsupported tintColor \"" + value + "\" was used.");
            return null;
        };
        this.logWarning = function (message) {
            console && console.warn && console.warn(message);
        };
    }
    Object.defineProperty(BbAvatar.prototype, "src", {
        get: function () {
            return this._src;
        },
        set: function (value) {
            this._src = value;
            if (this._src) {
                this.state$.next('image');
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbAvatar.prototype, "title", {
        get: function () {
            return this._title;
        },
        set: function (value) {
            // Set the title and the initials.
            this._title = value || '!';
            this._initials = this.getInitials(this._title);
            // Generate the colors based on the name.
            var _a = this.toRGb(this._title), r = _a.r, g = _a.g, b = _a.b;
            this.setColors(r, g, b);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbAvatar.prototype, "tintColor", {
        set: function (value) {
            var result = this.parseColorFormat(value);
            if (!result) {
                return;
            }
            var r = result.r, g = result.g, b = result.b;
            this.setColors(r, g, b);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbAvatar.prototype, "color", {
        get: function () {
            return this._color;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbAvatar.prototype, "backgroundColor", {
        get: function () {
            return this._backgroundColor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbAvatar.prototype, "initials", {
        get: function () {
            return this._initials;
        },
        enumerable: true,
        configurable: true
    });
    BbAvatar.prototype.setColors = function (red, green, blue) {
        this._color = "rgb(" + red + "," + green + "," + blue + ")";
        this._backgroundColor = "rgba(" + red + "," + green + "," + blue + ", 0.25)";
    };
    __decorate([
        Input()
    ], BbAvatar.prototype, "size", void 0);
    __decorate([
        Input()
    ], BbAvatar.prototype, "src", null);
    __decorate([
        Input(),
        HostBinding('attr.title')
    ], BbAvatar.prototype, "title", null);
    __decorate([
        Input()
    ], BbAvatar.prototype, "tintColor", null);
    BbAvatar = __decorate([
        Component({
            selector: 'bb-avatar',
            template: "<ng-container [ngSwitch]=\"state$ | async\">\r\n    <!-- Image. -->\r\n    <ng-container *ngSwitchCase=\"'image'\">\r\n        <img [src]=\"src\"\r\n             [alt]=\"initials\"\r\n             (error)=\"state$?.next('initials')\"\r\n             class=\"bb-avatar-image\">\r\n    </ng-container>\r\n\r\n    <!-- Default initials fallback. -->\r\n    <ng-container *ngSwitchDefault>\r\n        {{ initials }}\r\n    </ng-container>\r\n</ng-container>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            preserveWhitespaces: false,
            host: {
                'class': 'bb-avatar',
                'role': 'img',
                '[style.width.px]': 'size',
                '[style.height.px]': 'size',
                '[style.font-size.px]': 'size / 2',
                '[style.color]': 'color',
                '[style.background-color]': 'backgroundColor'
            },
            styles: [".bb-avatar{line-height:1;font-size:18px;overflow:hidden;font-weight:500;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;position:relative;border-radius:50%;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;-ms-flex-pack:center;justify-content:center}.bb-avatar.rounded{border-radius:4px}.bb-avatar.square{border-radius:0}.bb-avatar-image{width:100%;height:100%;-o-object-fit:cover;object-fit:cover;position:absolute}"]
        })
    ], BbAvatar);
    return BbAvatar;
}());

var BbFileSize = /** @class */ (function () {
    function BbFileSize() {
        this.format = function (value, decimals) {
            if (decimals === void 0) { decimals = 2; }
            if (value === 0) {
                return '0 Bytes';
            }
            var k = 1024;
            var dm = decimals < 0 ? 0 : decimals;
            var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
            var index = Math.floor(Math.log(value) / Math.log(k));
            return parseFloat((value / Math.pow(k, index)).toFixed(dm)) + ' ' + sizes[index];
        };
    }
    BbFileSize.prototype.transform = function (value) {
        return this.format(value);
    };
    BbFileSize = __decorate([
        Pipe({
            name: 'bbFileSize'
        })
    ], BbFileSize);
    return BbFileSize;
}());

var BbIcon = /** @class */ (function () {
    function BbIcon(_registry) {
        this._registry = _registry;
        // Data.
        this._src = null;
        // State.
        this.state$ = new BehaviorSubject(null);
        this.alt = null;
        this.size = 24;
        this._ariaHidden = true;
        this.warn = function (content) {
            return console && console.warn && console.warn(content);
        };
    }
    Object.defineProperty(BbIcon.prototype, "src", {
        get: function () {
            return this._src;
        },
        set: function (newValue) {
            this.state$.next(this.convert(newValue));
            this._src = newValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbIcon.prototype, "ariaHidden", {
        get: function () {
            return this._ariaHidden;
        },
        set: function (newValue) {
            this._ariaHidden = coerceBooleanProperty(newValue);
        },
        enumerable: true,
        configurable: true
    });
    BbIcon.prototype.convert = function (src) {
        var _this = this;
        var _a = __read((src || '').split(':'), 2), namespace = _a[0], value = _a[1];
        var onError = function () {
            _this.warn("ElementsModule: The icon with src \"" + src + "\" could not be rendered correctly; Did you put a correct src attribute?");
            return { type: 'unknown' };
        };
        if (!namespace || !value || !this._registry) {
            return onError();
        }
        // Try to find a image icon.
        var imageIcon = this.findImageIcon(namespace, value);
        if (imageIcon) {
            return imageIcon;
        }
        // Try to find a font icon.
        var fontIcon = this.findFontIcon(namespace, value);
        if (fontIcon) {
            return fontIcon;
        }
        // Return the on error function.
        return onError();
    };
    BbIcon.prototype.findImageIcon = function (namespace, value) {
        var data = this._registry.find(function (item) { return item['namespace'] === namespace && item['name'] === value; });
        if (!data) {
            return null;
        }
        return { type: data['type'], src: data['src'] };
    };
    BbIcon.prototype.findFontIcon = function (namespace, content) {
        var data = this._registry.find(function (item) { return item['namespace'] === namespace; });
        if (!data) {
            return null;
        }
        var isInline = data['inline'] || false;
        if (isInline) {
            return { type: data['type'], className: "" + data['className'] + content };
        }
        return { type: data['type'], className: data['className'], content: content };
    };
    BbIcon.ctorParameters = function () { return [
        { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [ELEMENTS_ICONS,] }] }
    ]; };
    __decorate([
        Input()
    ], BbIcon.prototype, "alt", void 0);
    __decorate([
        Input()
    ], BbIcon.prototype, "size", void 0);
    __decorate([
        Input()
    ], BbIcon.prototype, "src", null);
    __decorate([
        Input()
    ], BbIcon.prototype, "ariaHidden", null);
    BbIcon = __decorate([
        Component({
            selector: 'bb-icon',
            template: "<ng-container *ngIf=\"state$ | async as data\"\r\n              [ngSwitch]=\"data?.type\">\r\n    <i *ngSwitchCase=\"'font'\"\r\n       [class]=\"data?.className\">\r\n        {{ data?.content }}\r\n    </i>\r\n    <img *ngSwitchCase=\"'image'\"\r\n         [src]=\"data?.src\"\r\n         [alt]=\"alt\">\r\n    <svg *ngSwitchCase=\"'unknown'\"\r\n         viewBox=\"0 0 1 1\">\r\n        <rect x=\"0\" y=\"0\" width=\"1\" height=\"1\" fill=\"#c5c5c5\"></rect>\r\n    </svg>\r\n</ng-container>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            host: {
                'class': 'bb-icon',
                '[attr.aria-hidden]': 'ariaHidden',
                '[style.width.px]': 'size',
                '[style.height.px]': 'size',
                '[style.min-width.px]': 'size',
                '[style.min-height.px]': 'size'
            },
            preserveWhitespaces: false,
            styles: [".bb-icon{display:-ms-flexbox;display:flex;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center}.bb-icon>img,.bb-icon>svg{width:100%;height:100%;display:-ms-flexbox;display:flex}"]
        }),
        __param(0, Optional()), __param(0, Inject(ELEMENTS_ICONS))
    ], BbIcon);
    return BbIcon;
}());

var BbTag = /** @class */ (function () {
    function BbTag() {
        // Outputs.
        this.onClose = new EventEmitter();
    }
    __decorate([
        Output()
    ], BbTag.prototype, "onClose", void 0);
    BbTag = __decorate([
        Component({
            selector: 'bb-tag',
            template: "<ng-content></ng-content>\r\n\r\n<button *ngIf=\"onClose?.observers?.length > 0\"\r\n        (click)=\"onClose.emit()\"\r\n        type=\"button\">\r\n    Close\r\n</button>",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            preserveWhitespaces: false,
            host: { 'class': 'bb-tag' },
            styles: [".bb-tag{cursor:default;padding:0 7px;font-size:12px;list-style:none;line-height:20px;border-radius:4px;white-space:nowrap;display:inline-block;color:rgba(0,0,0,.65);border:1px solid #d9d9d9;background-color:#fafafa;font-variant:tabular-nums;font-feature-settings:\"tnum\"}.bb-tag:not(:last-child){margin-right:8px}"]
        })
    ], BbTag);
    return BbTag;
}());

var DECLARATIONS_EXPORTS = [
    // Directives.
    BbTemplate,
    BbFormError,
    BbFormSubmit,
    BbInput,
    BbSelect,
    BbFocus,
    BbFocusTrap,
    BbAutosize,
    BbPrefix,
    BbSuffix,
    // Pipes.
    BbFileSize,
    BbFileImage,
    BbRelativeTime,
    // Components.
    BbButton,
    BbAnchor,
    BbSpinner,
    BbIcon,
    BbFormControl,
    BbFormGroup,
    BbCheckbox,
    BbImagePicker,
    BbDropdown,
    BbDropdownItem,
    BbTag,
    BbAvatar,
    BbFilePicker
];
var ElementsModule = /** @class */ (function () {
    function ElementsModule() {
    }
    ElementsModule_1 = ElementsModule;
    ElementsModule.forRoot = function () {
        return {
            ngModule: ElementsModule_1,
            providers: [
                { provide: ELEMENTS_ERRORS, useFactory: getErrors, multi: true },
                ELEMENTS_MATERIAL_ICON
            ]
        };
    };
    var ElementsModule_1;
    ElementsModule = ElementsModule_1 = __decorate([
        NgModule({
            imports: [CommonModule, LocalizeModule],
            declarations: __spread(DECLARATIONS_EXPORTS),
            exports: __spread(DECLARATIONS_EXPORTS)
        })
    ], ElementsModule);
    return ElementsModule;
}());
var ɵ0 = { namespace: 'material', className: 'material-icons', type: 'font' };
var ELEMENTS_MATERIAL_ICON = {
    provide: ELEMENTS_ICONS,
    useValue: ɵ0,
    multi: true
};
function getErrors() {
    var data = {
        unknown: function () { return 'form-control-errors.unknown'; },
        required: function () { return 'form-control-errors.required'; },
        pattern: function () { return 'form-control-errors.pattern'; },
        email: function () { return 'form-control-errors.email'; },
        emailTaken: function () { return 'form-control-errors.emailTaken'; },
        passwordMatch: function () { return 'form-control-errors.passwordMatch'; },
        confirm: function () { return 'form-control-errors.confirm'; },
        fullNumber: function () { return 'form-control-errors.fullNumber'; },
        min: function (_a) {
            var min = _a.min;
            return ({ token: 'form-control-errors.min', data: { min: min } });
        },
        max: function (_a) {
            var max = _a.max;
            return ({ token: 'form-control-errors.max', data: { max: max } });
        },
        maxFileSize: function (_a) {
            var maxSize = _a.maxSize;
            return ({ token: 'form-control-errors.maxFileSize', data: { maxSize: maxSize } });
        },
        invalidFileType: function () { return 'form-control-errors.invalidFileType'; },
        minlength: function (_a) {
            var requiredLength = _a.requiredLength, actualLength = _a.actualLength;
            return ({
                token: 'form-control-errors.minlength',
                data: { requiredLength: requiredLength, actualLength: actualLength }
            });
        },
        maxlength: function (_a) {
            var requiredLength = _a.requiredLength, actualLength = _a.actualLength;
            return ({
                token: 'form-control-errors.maxlength',
                data: { requiredLength: requiredLength, actualLength: actualLength }
            });
        }
    };
    return data;
}

/**
 * Generated bundle index. Do not edit.
 */

export { BbAnchor, BbAutosize, BbAvatar, BbButton, BbCheckbox, BbDropdown, BbDropdownItem, BbFileImage, BbFilePicker, BbFileSize, BbFocus, BbFocusTrap, BbFormControl, BbFormError, BbFormGroup, BbFormSubmit, BbIcon, BbImagePicker, BbInput, BbPrefix, BbRelativeTime, BbSelect, BbSpinner, BbSuffix, BbTag, BbTemplate, ELEMENTS_ERRORS, ELEMENTS_ICONS, ELEMENTS_MATERIAL_ICON, ElementsModule, getErrors, ɵ0 };
//# sourceMappingURL=bravobit-bb-foundation-elements.js.map
