import { __decorate, __read, __extends, __param, __spread } from 'tslib';
import { Directive, ContentChildren, TemplateRef, Input, Component, ChangeDetectionStrategy, ViewEncapsulation, Optional, Host, Attribute, HostListener, forwardRef, NgModule } from '@angular/core';
import { DataSourceRequest } from '@bravobit/bb-foundation/api';
import { tap, map, catchError, debounceTime, distinctUntilChanged, startWith, share } from 'rxjs/operators';
import { of, combineLatest, BehaviorSubject } from 'rxjs';
import { FormControl, FormGroup, NG_VALUE_ACCESSOR, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { mixinDisabled } from '@bravobit/bb-foundation';
import { ElementsModule } from '@bravobit/bb-foundation/elements';
import { LocalizeModule } from '@bravobit/bb-foundation/localize';
import { CommonModule } from '@angular/common';

var BbTableColumnDef = /** @class */ (function () {
    function BbTableColumnDef() {
    }
    BbTableColumnDef = __decorate([
        Directive({
            selector: '[bbTableColumnDef]'
        })
    ], BbTableColumnDef);
    return BbTableColumnDef;
}());
var BbTable = /** @class */ (function () {
    function BbTable() {
        var _this = this;
        // Inputs.
        this.identifier = null;
        this.queryFields = [];
        this.sortingStrategy = 'single';
        this.header = true;
        this.footer = true;
        // Sizes.
        this._sizes = [25, 50, 75, 100];
        // Optimizations.
        this.trackByFn = function (index, item) { return _this.identifier ? item[_this.identifier] : index; };
        this.trackByValue = function (index, item) { return item; };
        this.queryControl = new FormControl('');
    }
    Object.defineProperty(BbTable.prototype, "sizes", {
        get: function () {
            return this._sizes;
        },
        set: function (newValue) {
            this.updateSizes(newValue);
        },
        enumerable: true,
        configurable: true
    });
    BbTable.prototype.ngOnInit = function () {
        this.composeForm();
        this.watchDataChanges();
    };
    BbTable.prototype.ngOnDestroy = function () {
        this._subscription && this._subscription.unsubscribe();
    };
    Object.defineProperty(BbTable.prototype, "sortControl", {
        get: function () {
            var _a;
            return (_a = this.form) === null || _a === void 0 ? void 0 : _a.get('sort');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbTable.prototype, "pageNumberControl", {
        get: function () {
            var _a;
            return (_a = this.form) === null || _a === void 0 ? void 0 : _a.get('pageNumber');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbTable.prototype, "pageSizeControl", {
        get: function () {
            var _a;
            return (_a = this.form) === null || _a === void 0 ? void 0 : _a.get('pageSize');
        },
        enumerable: true,
        configurable: true
    });
    BbTable.prototype.refresh = function () {
        return this.request.update();
    };
    BbTable.prototype.updateSizes = function (newValue) {
        // Validate the sizes has at least one value.
        if (newValue.length <= 0) {
            throw new Error('Cannot set an empty array as the sizes.');
        }
        // Set the new size.
        this._sizes = newValue;
        // Validate that the form exists already.
        if (!this.form) {
            return;
        }
        // Check if the current selected value is
        // in the new sizes array.
        var pageSizeControl = this.form.get('pageSize');
        if (this._sizes.includes(pageSizeControl.value)) {
            return;
        }
        // Set the first value in the array as
        // the new value.
        var firstValue = this._sizes[0];
        pageSizeControl.setValue(firstValue);
    };
    BbTable.prototype.composeForm = function () {
        // Setup the form.
        this.form = new FormGroup({
            pageSize: new FormControl(this._sizes[0]),
            pageNumber: new FormControl(1),
            sort: new FormGroup({})
        });
    };
    BbTable.prototype.watchDataChanges = function () {
        var _this = this;
        // Create a new request.
        this.request = new DataSourceRequest();
        this.isLoading$ = this.request.loading;
        if (!this.dataSource) {
            this.data$ = of({ state: 'error_datasource' }).pipe(tap(function () { return _this.request.stopLoading(); }));
            return;
        }
        // Pass the request to the data source.
        this.data$ = this.dataSource.handle(this.request).pipe(tap(function () { return _this.request.stopLoading(); }), map(function (result) {
            var count = (result && result.data && result.data.length) || 0;
            if (count <= 0) {
                return { state: 'empty', result: result };
            }
            return { state: 'success', result: result };
        }), catchError(function (error) {
            _this.request.stopLoading();
            var states = {
                401: 'error_unauthorized',
                403: 'error_forbidden',
                404: 'error_not_found',
                500: 'error_server'
            };
            return of({ state: states[error.status] || 'error_unknown', statusCode: error.status });
        }));
        // Subscribe to the data changes.
        this.subscribeToChanges();
    };
    BbTable.prototype.subscribeToChanges = function () {
        var _this = this;
        var form$ = this.getFormChanges();
        var query$ = this.getQueryChanges();
        this._subscription = combineLatest([form$, query$]).subscribe(function (_a) {
            var _b = __read(_a, 2), form = _b[0], query = _b[1];
            var pageNumber = form.pageNumber, pageSize = form.pageSize;
            var sort = Object.keys(form.sort)
                .map(function (name) { return ({ name: name, direction: form.sort[name] }); })
                .filter(function (item) { return ['asc', 'desc'].includes(item.direction); });
            _this.request.startLoading();
            return _this.request.update({
                sort: sort,
                query: query,
                pageSize: pageSize,
                pageNumber: pageNumber,
                queryFields: _this.queryFields
            });
        });
    };
    BbTable.prototype.getQueryChanges = function () {
        return this.queryControl.valueChanges.pipe(debounceTime(400), distinctUntilChanged(), startWith(this.queryControl.value));
    };
    BbTable.prototype.getFormChanges = function () {
        return this.form.valueChanges.pipe(startWith(this.form.value));
    };
    __decorate([
        ContentChildren(BbTableColumnDef, { read: TemplateRef })
    ], BbTable.prototype, "tableColumnTemplates", void 0);
    __decorate([
        Input()
    ], BbTable.prototype, "identifier", void 0);
    __decorate([
        Input()
    ], BbTable.prototype, "dataSource", void 0);
    __decorate([
        Input()
    ], BbTable.prototype, "queryFields", void 0);
    __decorate([
        Input()
    ], BbTable.prototype, "sortingStrategy", void 0);
    __decorate([
        Input()
    ], BbTable.prototype, "header", void 0);
    __decorate([
        Input()
    ], BbTable.prototype, "footer", void 0);
    __decorate([
        Input()
    ], BbTable.prototype, "sizes", null);
    BbTable = __decorate([
        Component({
            selector: 'bb-table',
            template: "<!--\r\n    This is the header of the table. It can be a user\r\n    defined custom header or just the default header.\r\n-->\r\n\r\n<header *ngIf=\"(header === true ? headerTemplate : header) as template\"\r\n        class=\"bb-table-top-actions\">\r\n    <ng-container *ngTemplateOutlet=\"template\"></ng-container>\r\n</header>\r\n\r\n<!--\r\n    This is the table that contains the\r\n    rows and columns.\r\n-->\r\n\r\n<div class=\"bb-table-wrapper\">\r\n    <div *ngIf=\"isLoading$ | async\"\r\n         class=\"bb-table-progress\">\r\n        <div class=\"bb-table-indeterminate\"></div>\r\n    </div>\r\n    <table class=\"bb-table-container\">\r\n        <thead>\r\n            <tr>\r\n                <ng-content select=\"bb-table-header-cell\"></ng-content>\r\n            </tr>\r\n        </thead>\r\n        <tbody *ngIf=\"data$ | async as data; else isLoadingTemplate\"\r\n               [ngSwitch]=\"data?.state\">\r\n            <ng-container *ngSwitchCase=\"'success'\">\r\n                <tr *ngFor=\"let item of data?.result?.data; trackBy: trackByFn\">\r\n                    <ng-container *ngFor=\"let template of tableColumnTemplates\">\r\n                        <ng-container *ngTemplateOutlet=\"template; context: {item: item}\"></ng-container>\r\n                    </ng-container>\r\n                </tr>\r\n            </ng-container>\r\n\r\n            <ng-container *ngSwitchCase=\"'empty'\">\r\n                <ng-container\r\n                    *ngTemplateOutlet=\"errorTemplate; context: data\"></ng-container>\r\n            </ng-container>\r\n\r\n            <ng-container *ngSwitchCase=\"'error_datasource'\">\r\n                <ng-container\r\n                    *ngTemplateOutlet=\"errorTemplate; context: data\"></ng-container>\r\n            </ng-container>\r\n\r\n            <ng-container *ngSwitchCase=\"'error_not_found'\">\r\n                <ng-container\r\n                    *ngTemplateOutlet=\"errorTemplate; context: data\"></ng-container>\r\n            </ng-container>\r\n\r\n            <ng-container *ngSwitchCase=\"'error_server'\">\r\n                <ng-container\r\n                    *ngTemplateOutlet=\"errorTemplate; context: data\"></ng-container>\r\n            </ng-container>\r\n\r\n            <ng-container *ngSwitchDefault>\r\n                <ng-container\r\n                    *ngTemplateOutlet=\"errorTemplate; context: data\"></ng-container>\r\n            </ng-container>\r\n        </tbody>\r\n    </table>\r\n</div>\r\n\r\n<footer *ngIf=\"(footer === true ? footerTemplate : footer) as template\"\r\n        class=\"bb-table-bottom-actions\">\r\n    <ng-container *ngTemplateOutlet=\"template\"></ng-container>\r\n</footer>\r\n\r\n<ng-template #isLoadingTemplate>\r\n    <tr>\r\n        <td [attr.colspan]=\"tableColumnTemplates?.length\"\r\n            class=\"bb-table-template\">\r\n            <div class=\"bb-table-template-content\">\r\n                {{ 'tables.loading' | bbLocalize }}\r\n            </div>\r\n        </td>\r\n    </tr>\r\n</ng-template>\r\n\r\n<!--\r\n    This is the template that will be used when the user has not\r\n    supplied a custom header template.\r\n-->\r\n\r\n<ng-template #headerTemplate>\r\n    <ng-content></ng-content>\r\n    <bb-form-control>\r\n        <input [formControl]=\"queryControl\"\r\n               [placeholder]=\"'tables.search' | bbLocalize\"\r\n               bbInput\r\n               autocomplete=\"off\">\r\n        <bb-icon *ngIf=\"queryControl?.value?.length <= 0; else closeButtonTemplate\"\r\n                 bbSuffix\r\n                 src=\"material:search\">\r\n        </bb-icon>\r\n    </bb-form-control>\r\n</ng-template>\r\n\r\n<!--\r\n    This is the template that will be used when the user has not\r\n    supplied a custom footer template.\r\n-->\r\n\r\n<ng-template #footerTemplate>\r\n    <bb-form-control *ngIf=\"sizes?.length > 1\">\r\n        <select [formControl]=\"pageSizeControl\"\r\n                bbInput\r\n                title=\"Sizes\">\r\n            <option *ngFor=\"let item of sizes; trackBy: trackByValue\"\r\n                    [ngValue]=\"item\">\r\n                {{ item }} / {{ 'tables.page' | bbLocalize }}\r\n            </option>\r\n        </select>\r\n    </bb-form-control>\r\n\r\n    <bb-table-pager [totalPages]=\"(data$ | async)?.result?.totalPages\"\r\n                    [formControl]=\"pageNumberControl\">\r\n    </bb-table-pager>\r\n</ng-template>\r\n\r\n<!--\r\n    This the template that will be used when the user\r\n    has filled in some data in the query control.\r\n-->\r\n\r\n<ng-template #closeButtonTemplate>\r\n    <button bbSuffix\r\n            (click)=\"queryControl?.patchValue('')\"\r\n            type=\"button\"\r\n            class=\"bb-table-progress-close-query\">\r\n        <bb-icon src=\"material:close\"></bb-icon>\r\n    </button>\r\n</ng-template>\r\n\r\n<ng-template #errorTemplate let-state=\"state\" let-status=\"statusCode\">\r\n    <tr>\r\n        <td [attr.colspan]=\"tableColumnTemplates?.length\"\r\n            class=\"bb-table-template\">\r\n            <div class=\"bb-table-template-content\">\r\n                <svg xmlns=\"http://www.w3.org/2000/svg\" width=\"64\" height=\"41\">\r\n                    <g fill=\"none\" fill-rule=\"evenodd\" transform=\"translate(0 1)\">\r\n                        <ellipse cx=\"32\" cy=\"33\" fill=\"#F5F5F5\" rx=\"32\" ry=\"7\"></ellipse>\r\n                        <g fill-rule=\"nonzero\" stroke=\"#D9D9D9\">\r\n                            <path\r\n                                d=\"M55 12.7605604L44.8543047 1.25739633C44.3674414.47382661 43.6558789 0 42.9067617 0H21.0932383c-.7491172 0-1.4606797.47395025-1.947543 1.25739633L9 12.7605604V22h46v-9.2394396z\"></path>\r\n                            <path fill=\"#FAFAFA\"\r\n                                  d=\"M41.6132813 15.9315c0-1.6056489.9936718-2.931266 2.2264179-2.9315H55v18.1371277C55 33.2589574 53.6793867 35 52.0504297 35H11.94957031C10.32052344 35 9 33.2588404 9 31.1371277V13h11.1603008c1.2327461 0 2.226418 1.3228085 2.226418 2.9284574v.0211809c0 1.6056489 1.0049921 2.9015426 2.2377382 2.9015426h14.751086c1.2327461 0 2.2377383-1.3078298 2.2377383-2.9134788V15.9315z\"></path>\r\n                        </g>\r\n                    </g>\r\n                </svg>\r\n                <span *ngIf=\"state as stateLabel\">{{ ('tables.' + stateLabel) | bbLocalize }}</span>\r\n                <span *ngIf=\"status as statusCode\"\r\n                      class=\"bb-table-template-content-code\">(HTTP {{ statusCode }})</span>\r\n            </div>\r\n        </td>\r\n    </tr>\r\n</ng-template>\r\n",
            host: { 'class': 'bb-table' },
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            preserveWhitespaces: false,
            styles: [".bb-table{width:100%;display:block;box-shadow:0 6px 6px -6px rgba(0,0,0,.1)}.bb-table.plain{box-shadow:none}.bb-table.plain>.bb-table-wrapper{border-top-left-radius:0;border-top-right-radius:0}.bb-table-wrapper{overflow-x:auto;position:relative;background-color:#fff;border-top-left-radius:4px;border-top-right-radius:4px}.bb-table-container{width:100%}.bb-table-container td{border-bottom:1px solid #d8d8d8}.bb-table-top-actions{display:-ms-flexbox;display:flex;-ms-flex-wrap:wrap;flex-wrap:wrap;-ms-flex-direction:column;flex-direction:column;padding:20px 20px 10px;background-color:#fff;-ms-flex-pack:end;justify-content:flex-end;border-top-left-radius:4px;border-top-right-radius:4px;border-bottom:1px solid #d8d8d8}.bb-table-top-actions>*{margin-left:0;margin-bottom:10px}.bb-table-top-actions+.bb-table-wrapper{border-radius:0}.bb-table-bottom-actions{height:60px;display:-ms-flexbox;display:flex;padding:0 20px;overflow-x:auto;-ms-flex-align:center;align-items:center;background-color:#fff;border-bottom-left-radius:4px;border-bottom-right-radius:4px}.bb-table-template{border:none;background-color:#f5f5f5;border-bottom:1px solid #d8d8d8}.bb-table-template-content{padding:20px;display:-ms-flexbox;display:flex;color:#3d464d;font-size:14px;line-height:1.2;font-weight:400;min-height:140px;text-align:center;-ms-flex-align:center;align-items:center;-ms-flex-direction:column;flex-direction:column;-ms-flex-pack:center;justify-content:center}.bb-table-template-content>svg{margin-bottom:12px}.bb-table-template-content-code{opacity:.5;margin-top:5px;display:inline-block}.bb-table-progress{top:0;left:0;right:0;opacity:0;width:100%;height:4px;display:block;overflow:hidden;position:absolute;background-color:rgba(47,64,139,.3);-webkit-animation:250ms cubic-bezier(0,0,.2,1) .1s forwards fadeIn;animation:250ms cubic-bezier(0,0,.2,1) .1s forwards fadeIn}.bb-table-progress .bb-table-indeterminate{background-color:#2f408b}.bb-table-progress .bb-table-indeterminate:before{top:0;left:0;bottom:0;content:\"\";position:absolute;will-change:left,right;background-color:inherit;-webkit-animation:2.1s cubic-bezier(.65,.815,.735,.395) infinite indeterminate;animation:2.1s cubic-bezier(.65,.815,.735,.395) infinite indeterminate}.bb-table-progress .bb-table-indeterminate:after{top:0;left:0;bottom:0;content:\"\";position:absolute;-webkit-animation-delay:1.15s;animation-delay:1.15s;will-change:left,right;background-color:inherit;-webkit-animation:2.1s cubic-bezier(.165,.84,.44,1) infinite indeterminate-short;animation:2.1s cubic-bezier(.165,.84,.44,1) infinite indeterminate-short}.bb-table-progress-close-query{padding:0;border:none;border-radius:4px;background-color:transparent;transition:background-color 250ms cubic-bezier(0,0,.2,1)}.bb-table-progress-close-query:hover{background-color:rgba(0,0,0,.08)}.bb-table-progress-close-query:focus{background-color:rgba(0,0,0,.12)}.bb-table-progress-close-query:active{background-color:rgba(0,0,0,.16)}@-webkit-keyframes fadeIn{from{opacity:0}to{opacity:1}}@keyframes fadeIn{from{opacity:0}to{opacity:1}}@-webkit-keyframes indeterminate{0%{left:-35%;right:100%}100%,60%{left:100%;right:-90%}}@keyframes indeterminate{0%{left:-35%;right:100%}100%,60%{left:100%;right:-90%}}@-webkit-keyframes indeterminate-short{0%{left:-200%;right:100%}100%,60%{left:107%;right:-8%}}@keyframes indeterminate-short{0%{left:-200%;right:100%}100%,60%{left:107%;right:-8%}}@media only screen and (min-width:768px){.bb-table-top-actions{-ms-flex-direction:row;flex-direction:row}.bb-table-top-actions>:not(:first-child){margin-left:10px;margin-bottom:10px}}"]
        })
    ], BbTable);
    return BbTable;
}());

var BbTableCell = /** @class */ (function () {
    function BbTableCell() {
    }
    BbTableCell = __decorate([
        Component({
            selector: 'bb-table-cell',
            template: "<div>\r\n    <ng-content></ng-content>\r\n</div>",
            changeDetection: ChangeDetectionStrategy.OnPush,
            host: { 'class': 'bb-table-cell' },
            encapsulation: ViewEncapsulation.None,
            preserveWhitespaces: false,
            styles: [".bb-table-cell{color:#848f99;font-size:14px;text-align:left;line-height:1.2;font-weight:400;padding:12px 20px;display:table-cell;word-wrap:break-word;vertical-align:middle;background-color:#f5f5f5;border-bottom:1px solid #d8d8d8}.bb-table-cell:not(:last-child){border-right:1px solid #d8d8d8}.bb-table-cell.fit{width:1%;white-space:nowrap}.bb-table-cell.left{text-align:left}.bb-table-cell.center{text-align:center}.bb-table-cell.right{text-align:right}"]
        })
    ], BbTableCell);
    return BbTableCell;
}());

var BbTableHeaderCellBase = /** @class */ (function () {
    function BbTableHeaderCellBase() {
    }
    return BbTableHeaderCellBase;
}());
var BbTableHeaderCellMixinBase = mixinDisabled(BbTableHeaderCellBase);
var BbTableHeaderCell = /** @class */ (function (_super) {
    __extends(BbTableHeaderCell, _super);
    function BbTableHeaderCell(_table, _name) {
        var _this = _super.call(this) || this;
        _this._table = _table;
        _this._name = _name;
        // Constants.
        _this._sortDirections = ['none', 'asc', 'desc'];
        _this._control = new FormControl(_this._sortDirections[0]);
        _this.logWarning = function (message) {
            console && console.warn && console.warn(message);
        };
        return _this;
    }
    Object.defineProperty(BbTableHeaderCell.prototype, "sort", {
        get: function () {
            return this._control.value;
        },
        // Sort.
        set: function (value) {
            // Check if the new value is a valid value.
            if (!this._sortDirections.includes(value)) {
                return;
            }
            // Set the value.
            this._control.setValue(value);
        },
        enumerable: true,
        configurable: true
    });
    BbTableHeaderCell.prototype.ngOnInit = function () {
        var _a, _b;
        // Validate that the name and table are set.
        if (!this._table || !this._name) {
            return;
        }
        // Check if the control already exists in the table.
        if ((_a = this._table.sortControl) === null || _a === void 0 ? void 0 : _a.contains(this._name)) {
            this.disabled = true;
            return this.logWarning("BbTableHeaderCell: The name \"" + this._name + "\" is not unique in the table; Control was not registered.");
        }
        // Register the header cell with the table.
        (_b = this._table.sortControl) === null || _b === void 0 ? void 0 : _b.addControl(this._name, this._control);
        // Get all the value changes.
        this.valueChanges$ = this._control.valueChanges.pipe(startWith(this._control.value));
    };
    BbTableHeaderCell.prototype.ngOnDestroy = function () {
        var _a, _b, _c, _d;
        // Validate that the name and table are set.
        if (!this._table || !this._name) {
            return;
        }
        // Un-register the header cell with the table.
        if ((_b = (_a = this._table) === null || _a === void 0 ? void 0 : _a.sortControl) === null || _b === void 0 ? void 0 : _b.contains(this._name)) {
            (_d = (_c = this._table) === null || _c === void 0 ? void 0 : _c.sortControl) === null || _d === void 0 ? void 0 : _d.removeControl(this._name);
        }
    };
    Object.defineProperty(BbTableHeaderCell.prototype, "isDisabled", {
        get: function () {
            return this.disabled || !this._name;
        },
        enumerable: true,
        configurable: true
    });
    BbTableHeaderCell.prototype.onClick = function () {
        var _this = this;
        // Check if the header cell is disabled.
        if (this.isDisabled) {
            return;
        }
        // Find the index of the current sort and
        // create the new index.
        var index = this._sortDirections.findIndex(function (item) { return item === _this._control.value; });
        var newIndex = (index + 1) % this._sortDirections.length;
        var newSort = this._sortDirections[newIndex];
        // Set the new sort.
        return this.updateTableAndHeaderCell(newSort);
    };
    BbTableHeaderCell.prototype.updateTableAndHeaderCell = function (value) {
        // Check if the new value is a valid value.
        if (!this._sortDirections.includes(value)) {
            return;
        }
        // Validate the table exists.
        if (!this._table) {
            return this._control.setValue(value);
        }
        // Handle the single sorting strategy.
        if (this._table.sortingStrategy === 'single') {
            return this.handleSingleSortingStrategy(value);
        }
        // Default case when no other strategy
        // is used.
        return this._control.setValue(value);
    };
    BbTableHeaderCell.prototype.handleSingleSortingStrategy = function (value) {
        var _this = this;
        var _a, _b;
        // Get the sort control.
        var control = this._table.sortControl;
        // Set each value to false since we only want to
        // sort on one property at a time with this strategy.
        var newValue = Object.keys(control.controls).reduce(function (previous, current) {
            previous[current] = current === _this._name ? value : 'none';
            return previous;
        }, {});
        // Patch the entire sort value.
        (_b = (_a = this._table) === null || _a === void 0 ? void 0 : _a.sortControl) === null || _b === void 0 ? void 0 : _b.patchValue(newValue);
    };
    BbTableHeaderCell.ctorParameters = function () { return [
        { type: BbTable, decorators: [{ type: Optional }, { type: Host }] },
        { type: String, decorators: [{ type: Optional }, { type: Attribute, args: ['name',] }] }
    ]; };
    __decorate([
        Input()
    ], BbTableHeaderCell.prototype, "sort", null);
    __decorate([
        HostListener('click')
    ], BbTableHeaderCell.prototype, "onClick", null);
    BbTableHeaderCell = __decorate([
        Component({
            selector: 'bb-table-header-cell',
            template: "<!-- The content. -->\r\n<ng-content></ng-content>\r\n\r\n<!-- The indicator for the sort direction. -->\r\n<ng-container *ngIf=\"valueChanges$ | async as value\">\r\n    <button *ngIf=\"!isDisabled\"\r\n            [class.asc]=\"value === 'asc'\"\r\n            [class.desc]=\"value === 'desc'\"\r\n            class=\"bb-table-header-cell-toggle\"\r\n            type=\"button\"></button>\r\n</ng-container>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            host: {
                'class': 'bb-table-header-cell',
                '[class.disabled]': 'isDisabled'
            },
            inputs: ['disabled'],
            encapsulation: ViewEncapsulation.None,
            preserveWhitespaces: false,
            styles: [".bb-table-header-cell{height:50px;color:#212529;cursor:pointer;font-size:16px;font-weight:500;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;padding:10px 20px;position:relative;display:table-cell;vertical-align:middle;border-bottom:1px solid #d8d8d8}.bb-table-header-cell:active>.bb-table-header-cell-toggle{background-color:rgba(0,0,0,.08)}.bb-table-header-cell:not(.disabled){padding-right:52px}.bb-table-header-cell.fit{width:1%;white-space:nowrap}.bb-table-header-cell.left{text-align:left}.bb-table-header-cell.center{text-align:center}.bb-table-header-cell.right{text-align:right}.bb-table-header-cell.disabled{cursor:default;pointer-events:none}.bb-table-header-cell-toggle{top:50%;right:14px;width:22px;border:none;height:22px;margin-left:auto;position:absolute;border-radius:4px;-ms-transform:translateY(-50%);transform:translateY(-50%);background-position:center;background-repeat:no-repeat;background-color:transparent;transition:background-color .2s cubic-bezier(0,0,.2,1);background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E')}.bb-table-header-cell-toggle.desc{background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"black\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E')}.bb-table-header-cell-toggle.asc{background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"black\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E')}"]
        }),
        __param(0, Optional()), __param(0, Host()),
        __param(1, Optional()), __param(1, Attribute('name'))
    ], BbTableHeaderCell);
    return BbTableHeaderCell;
}(BbTableHeaderCellMixinBase));

var BbTablePagerBase = /** @class */ (function () {
    function BbTablePagerBase() {
    }
    return BbTablePagerBase;
}());
var BbTablePagerMixinBase = mixinDisabled(BbTablePagerBase);
var BbTablePager = /** @class */ (function (_super) {
    __extends(BbTablePager, _super);
    function BbTablePager() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // Inputs.
        _this.amountOfButtons = 5;
        _this.showArrowButtons = true;
        // Methods.
        _this.onChange = function () { return ({}); };
        _this.onTouched = function () { return ({}); };
        _this.trackByIndex = function (index, item) { return item.page; };
        // Data.
        _this._totalPages$ = new BehaviorSubject(1);
        _this._pageNumber$ = new BehaviorSubject(1);
        return _this;
    }
    BbTablePager_1 = BbTablePager;
    BbTablePager.prototype.ngOnInit = function () {
        var _this = this;
        // Validate the previous button is disabled.
        this.isPreviousDisabled$ = this._pageNumber$.pipe(map(function (pageNumber) { return pageNumber === 1; }));
        // Validate the next button is disabled.
        this.isNextDisabled$ = combineLatest([this._pageNumber$, this._totalPages$]).pipe(map(function (_a) {
            var _b = __read(_a, 2), pageNumber = _b[0], totalPages = _b[1];
            return pageNumber === totalPages;
        }));
        // Create the list with page numbers.
        this.list$ = combineLatest([this._pageNumber$, this._totalPages$]).pipe(map(function (_a) {
            var _b = __read(_a, 2), pageNumber = _b[0], totalPages = _b[1];
            return _this.filterActivePages(pageNumber, totalPages);
        }), share());
    };
    Object.defineProperty(BbTablePager.prototype, "pageNumber", {
        get: function () {
            return this._pageNumber$.getValue();
        },
        set: function (newValue) {
            this.update(newValue);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BbTablePager.prototype, "totalPages", {
        get: function () {
            return this._totalPages$.getValue();
        },
        set: function (newValue) {
            var newTotalPages = Math.max(1, newValue);
            this._totalPages$.next(newTotalPages);
        },
        enumerable: true,
        configurable: true
    });
    BbTablePager.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    BbTablePager.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    BbTablePager.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    BbTablePager.prototype.writeValue = function (newValue) {
        this.update(newValue, false);
    };
    BbTablePager.prototype.goToPrevious = function () {
        var newValue = this._pageNumber$.getValue() - 1;
        return this.update(newValue);
    };
    BbTablePager.prototype.goToNext = function () {
        var newValue = this._pageNumber$.getValue() + 1;
        return this.update(newValue);
    };
    BbTablePager.prototype.goToIndex = function (index) {
        return this.update(index);
    };
    BbTablePager.prototype.update = function (pageNumber, notify) {
        if (notify === void 0) { notify = true; }
        // You can only update when not disabled.
        if (this.disabled) {
            return;
        }
        // Calculate the new value.
        var newValue = Math.min(this.totalPages, Math.max(1, pageNumber));
        // Don't do anything if the page number is the same.
        if (this._pageNumber$.getValue() === newValue) {
            return;
        }
        // Push the new value.
        this._pageNumber$.next(newValue);
        if (notify) {
            return this.onChange(pageNumber);
        }
    };
    BbTablePager.prototype.calculatePages = function (pageNumber) {
        var min = 1;
        var max = this.totalPages;
        var amount = Math.max(1, this.amountOfButtons);
        var data = Array(amount)
            .fill(0)
            .map(function (_, index) { return pageNumber + (index - 1); });
        // Validate if the minimum value less than
        // the first value.
        if (data[0] < min) {
            return data.map(function (item) { return item + 1; });
        }
        // Calculate the difference with the max
        // if this value is higher than 0 we should minus it.
        var lastValueInArray = data[data.length - 1];
        var differenceWithMax = lastValueInArray - max;
        if (differenceWithMax > 0) {
            return data.map(function (item) { return item - differenceWithMax; });
        }
        // Just return the default data.
        return data;
    };
    BbTablePager.prototype.filterActivePages = function (pageNumber, totalPages) {
        return this.calculatePages(pageNumber)
            .filter(function (page) { return page > 0 && page <= totalPages; })
            .map(function (page) { return ({ page: page, active: pageNumber === page }); });
    };
    var BbTablePager_1;
    __decorate([
        Input()
    ], BbTablePager.prototype, "amountOfButtons", void 0);
    __decorate([
        Input()
    ], BbTablePager.prototype, "showArrowButtons", void 0);
    __decorate([
        Input()
    ], BbTablePager.prototype, "pageNumber", null);
    __decorate([
        Input()
    ], BbTablePager.prototype, "totalPages", null);
    BbTablePager = BbTablePager_1 = __decorate([
        Component({
            selector: 'bb-table-pager',
            template: "<ul class=\"bb-pager-list\">\r\n    <!-- The previous page arrow button. -->\r\n    <li *ngIf=\"showArrowButtons\"\r\n        class=\"bb-pager-list-item\"\r\n        title=\"Previous page\">\r\n        <button [disabled]=\"isPreviousDisabled$ | async\"\r\n                (click)=\"goToPrevious()\"\r\n                class=\"bb-pager-list-button\"\r\n                type=\"button\">\r\n            <bb-icon src=\"material:keyboard_arrow_left\"></bb-icon>\r\n        </button>\r\n    </li>\r\n\r\n    <!-- The list of number buttons to navigate through the list. -->\r\n    <ng-container *ngIf=\"list$ | async as list\">\r\n        <li *ngFor=\"let item of list; trackBy: trackByIndex\"\r\n            [title]=\"'Page ' + item?.page\"\r\n            class=\"bb-pager-list-item\">\r\n            <button (click)=\"goToIndex(item?.page)\"\r\n                    [class.active]=\"item?.active\"\r\n                    class=\"bb-pager-list-button\"\r\n                    type=\"button\">\r\n                {{ item?.page }}\r\n            </button>\r\n        </li>\r\n    </ng-container>\r\n\r\n    <!-- The next page arrow button. -->\r\n    <li *ngIf=\"showArrowButtons\"\r\n        class=\"bb-pager-list-item\"\r\n        title=\"Next page\">\r\n        <button [disabled]=\"isNextDisabled$ | async\"\r\n                (click)=\"goToNext()\"\r\n                class=\"bb-pager-list-button\"\r\n                type=\"button\">\r\n            <bb-icon src=\"material:keyboard_arrow_right\"></bb-icon>\r\n        </button>\r\n    </li>\r\n</ul>\r\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            host: {
                'class': 'bb-table-pager',
                '[class.disabled]': 'disabled'
            },
            inputs: ['disabled'],
            encapsulation: ViewEncapsulation.None,
            preserveWhitespaces: false,
            providers: [
                { provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return BbTablePager_1; }), multi: true }
            ],
            styles: [".bb-table-pager{-ms-flex:1;flex:1;display:-ms-flexbox;display:flex;-ms-flex-align:center;align-items:center;-ms-flex-pack:end;justify-content:flex-end}.bb-table-pager.disabled{opacity:.5;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;pointer-events:none}.bb-pager-list{display:-ms-flexbox;display:flex}.bb-pager-list-item{margin-left:8px}.bb-pager-list-button{padding:0;color:#3d464d;line-height:1;min-width:32px;font-size:16px;-webkit-appearance:none;-moz-appearance:none;appearance:none;font-weight:500;min-height:32px;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;border-radius:4px;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;-ms-flex-pack:center;justify-content:center;border:1px solid #b6bbc1;transition-duration:.2s;background-color:transparent;transition-timing-function:cubic-bezier(0,0,.2,1);transition-property:color,border-color,opacity,background-color;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1)}.bb-pager-list-button:focus,.bb-pager-list-button:hover{background-color:rgba(0,0,0,.04)}.bb-pager-list-button.active,.bb-pager-list-button.active:hover{color:#2196f3;cursor:default;border-color:#2196f3;background-color:transparent}.bb-pager-list-button:disabled{opacity:.35;cursor:default;pointer-events:none}"]
        })
    ], BbTablePager);
    return BbTablePager;
}(BbTablePagerMixinBase));

var DECLARATIONS_EXPORTS = [
    BbTable,
    BbTableColumnDef,
    BbTableHeaderCell,
    BbTableCell
];
var TableModule = /** @class */ (function () {
    function TableModule() {
    }
    TableModule = __decorate([
        NgModule({
            imports: [CommonModule, FormsModule, ReactiveFormsModule, ElementsModule, LocalizeModule],
            declarations: __spread([BbTablePager], DECLARATIONS_EXPORTS),
            exports: __spread(DECLARATIONS_EXPORTS)
        })
    ], TableModule);
    return TableModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { BbTable, BbTableCell, BbTableColumnDef, BbTableHeaderCell, TableModule, BbTablePager as ɵa };
//# sourceMappingURL=bravobit-bb-foundation-table.js.map
